﻿/* --- FMfindRepl_Globals.jsx ------ UTF-8 -----------------------------------------------------
              Definition of global variables and arrays as part of the script object
Calling       There must not be any called functions here.
Comment       to be included in FMfindRepl.jsx
Reference     "JavaScript: The Good Parts" by Douglas Crockford [O’Reilly, 2008-05]
              is not a friend of global variables, hence I switched to object definition
History       2020-12-29
*/
  KLD_F.UItxt = {};                               // current texts by language switching

// === Window ID is set in the Panel functions. Therefore in Notify it must be tested,
//     whether the panel exists or not. If it exists, references to UI objects is possible
//     See PalFindRepl and Notify.
//     var declarations for the global windows variables left out to make it "superglobal"
//     See Klaus Göbel in private e-mail 2017-03-01

// === 'prototype' definitions ====================================================================
KLD_F.Variable = function (sName, sValue, oVar, bSystem, oTr, sPageType, sPageName) {
// Called by  GetVarNameList
  this.Name     =  sName;                         // Name of variable
  this.Value    =  sValue;                        // Contents of variable
  this.VarObj   =  oVar;                          // Var object in document
  this.IsSystem =  bSystem;                       // true for system variable
  this.TextRange=  oTr;                           // textrange of this occurance in the doc
} //--- end oVariable -----------------------------------------------------------------------------
KLD_F.UserString = function (sObjType, oObject, sString, oTL) {
// Called by  CollectUserStrings
  this.ObjType  = sObjType;
  this.ObjObj   = oObject;
  this.Ustring  = sString;
  this.TL       = oTL;
} //--- end oUserString ---------------------------------------------------------------------------
KLD_F.BookComponent = function (oDoc, sFileType, sName) {
// Called by  GetBookComponents
  this.Comp    = oDoc;
  this.sFType  = sFileType;                       // currently not used
  this.Name    = sName;
} //--- end BookComponent -------------------------------------------------------------------------

// === Constants (per definition global), global variables, arrays ================================
// const works only in ESTK. Invoking the script in FM I get "XXX_YYY_ZZ redeclared" for all these items
  KLD_F.APP_NAME    = "FMfindRepl";               // the name of the game
  KLD_F.APP_VERSION = "1.2";                      // current version of script (inc after publication)
  KLD_F.APP_LANG    = 'en';                       // set in SwitchUIlanguageX
  KLD_F.FM_ShortCut = true;                       // true:  Ctrl+f  is shortcut for FMfindRepl
                                                  // false: Ctrl+f  is shortcut for FM Find/Change
  KLD_F.sScriptDir  = undefined;                  // defined in FMfindRepl
  KLD_F.oHelpFile   = undefined;                  // defined in FMfindRepl
  KLD_F.oCurrentDoc = undefined;                  // Currently active document
  KLD_F.oCurrentBook= undefined;                  // Currently active book
  KLD_F.bBookActive = undefined;
  KLD_F.bNextCompOK = undefined;                  // true if no more found in current book comp.
  KLD_F.aoBookComps = [];                         // Array of book components
  KLD_F.kCurrentComp= null;                       // Index of current component
  KLD_F.oCurrentComp= undefined;                  // Current component
  KLD_F.nComponents = 0;                          // book components processed
  KLD_F.bDocActive  = undefined;
  KLD_F.LastDocID   = undefined;                  // used in Notify
  KLD_F.bRetrieved  = false;                      // toggle
  KLD_F.lstObjID    = undefined;                  // Obj id of last found object
  KLD_F.oFirstFindTR= undefined;                  // TR of first found item in this find cycle
  KLD_F.bFirstFind  = true;                       // Find cycle begins
  KLD_F.oLstObjFound= {};                         // for successive finds of special items
//                  .cell                            oCell of main object Tbl
//                  .empty                           found object is empty (TextLine)
//                  .grouped                         found object is within a group (TextLine)
//                  .object                          main object found (Flow, Tbl, TextLine)
//                  .row                             oRow of main object Tbl
  KLD_F.asColourNames= [];                        // Filled by GetColourNames from the XML file
  KLD_F.oReplObject = {}                          // object from the list of Replacing Object
//                  .type                            Type of object (mnemonic: MRKANY, ...)
//                  .name                            Format name, variable name
//                  .def                             defintion (content)
  KLD_F.iError      = 0;                          // -95: not found (anymore)
  KLD_F.aUserStrTypes = ["AFrame", "Tbl"];        // User strings are searched in these objects
  KLD_F.aoUserStrings = [];                       // Collection of objects UserString
  KLD_F.nUserStrings  = 0;                        // number of user strings collected
  KLD_F.jCurrentUStr  = 0;                        // this one is handled currently
  KLD_F.evBuffer      = "";                       // used by event handler ListSelector
//KLD_F.evSelect      = 0;                        // used by event handler ListSelector
//KLD_F.oBtnStatus    = {};                       // used in MaintainButtons
// ---------------------------------------------- UI lists ----------------------------------------
  KLD_F.xSettings   = {};                         // XML setting object
  KLD_F.iSettings   = 0;                          // item index in settings XML file
  KLD_F.sCloseName  = "ZZZZZZZZ";                 // name of setting saved at close of panel
  KLD_F.asSettingName = [];                       // filled by GetSettingsList
  KLD_F.sSettingName = undefined;                 // currently selected item
  KLD_F.asFindType  = [];                         // filled by FillFindTypeData
  KLD_F.asFType     = [];                         // Mnemonic ID, filled by FillFindTypeData
  KLD_F.asFindModes = [];                         // tandem to asFindType → Doc: Find type and options
  KLD_F.asReplModes = [];                         // tandem to asFindType → Doc: Find type and options
  KLD_F.asCatalogF  = [];                         // filled by GetCatalogF
  KLD_F.asCatalogC  = [];                         // filled by GetCatalogC
  KLD_F.aLstChars   = [];                         // filled by FillRegexLists
  KLD_F.aLstAnchor  = [];                         // filled by FillRegexLists
  KLD_F.aLstSynChars= [];                         // filled by FillRegexLists
  KLD_F.aLstMatch   = [];                         // filled by FillRegexLists
  KLD_F.aLstWhite   = [];                         // filled by FillRegexLists
  KLD_F.aLstSymbols = [];                         // filled by FillRegexLists
// ---------------------------------------------- Find and Replace parameters ---------------------
  KLD_F.iFindDefault = 0;                         // defined by FillFindTypeData
  KLD_F.findParmF    = undefined;                 // Constants.FS_xxx
  KLD_F.findParmV    = undefined;                 // Constants.FV_xxx
