﻿/* --- FMfindRepl_PalF.jsx ------ UTF-8 -----------------------------------------------------------
              Palette and Call-back functions, also ancillary function for the UI
Comment       to be included in FMfindRepl.jsx
History       2020-12-29
*/

KLD_F.PalFindRepl = function () { //===============================================================
/*            Set up the find/replace palette
Arguments     -
Returns       The visible palette window
Called by     FindReplInDoc, FindReplInBook
Calling       ClearInfoArea, SetReplButtons, SelectFindType, SelectedInCatalogA, SelectedInCatalogB,
              DisplaySettingsInfo, WarnBackwardsSearch, GetCatalogAny, PrepareBookProc, ButtonSave,
              ButtonRetrieve, OpenHelpFile, GetColourNames, FillFindTypeData, FillDlgList,
              GetSettingsList, BackupSettings, DisplayInfoF, GetBookComponents, GetTextOfSelection
Attention     To allow correct operation of th epanel after a close of it and re-invoking it,
              the window-ID must be defined here, not in the global module.
              Consequence: no reference to the panel-ID in the Notify function allowed.
History       2020-12-29
*/
  wPalF  = new Window("palette", undefined, undefined);  // title set in PalF
/*
  try { wPalF  = new Window("palette", KLD_F.UItxt.wPalF.@title, undefined); }
  catch (e) {
    KLD_F.Message ("E", KLD_F.UItxt.wPalF.@txt01, "PaletteF", e.message));
    return;
  }
*/
  var widthA, widthB;
  if (KLD_F.APP_LANG != "en") {                   // German, French have longer texts
    widthA = 120;
    widthB = 80;
  } else {
    widthA = 100;
    widthB = 100;
  }
  wPalF.text = KLD_F.APP_NAME + " [v" + KLD_F.APP_VERSION + "] — " + KLD_F.UItxt.wPalF.@title

  wPalF.preferredSize.width = 590;
  wPalF.orientation = "column";
  wPalF.alignChildren = ["left","top"];
  wPalF.spacing = 10;
  wPalF.margins = 10;

// -------------------------------------------------------------------------------- Group Find type
  wPalF.g01 = wPalF.add("group", undefined, undefined);
  wPalF.g01.orientation = "row";
  wPalF.g01.alignChildren = ["left","center"];
  wPalF.g01.spacing = 10;
  wPalF.g01.margins = 0;

  wPalF.g01.t1 = wPalF.g01.add("statictext", undefined, KLD_F.UItxt.wPalF.@g01t1);
  wPalF.g01.t1.preferredSize.width = 100;

  wPalF.g01.lstFindTypeF = wPalF.g01.add("dropdownlist", undefined, undefined);
  wPalF.g01.lstFindTypeF.helpTip = KLD_F.UItxt.wPalF.@g01lstFindTypeFhelpTip;
  wPalF.g01.lstFindTypeF.graphics.font = "dialog:12";
  wPalF.g01.lstFindTypeF.selection = KLD_F.iFindDefault;           // Text
  wPalF.g01.lstFindTypeF.preferredSize.width = 210;
  wPalF.g01.lstFindTypeF.addEventListener  ("keydown", KLD_F.ListSelector);

  wPalF.g01.lstCatalgF = wPalF.g01.add("dropdownlist", undefined, KLD_F.asCatalogF);
  wPalF.g01.lstCatalgF.graphics.font = "dialog:12";
  wPalF.g01.lstCatalgF.selection = 0;               // start with an empty entry
  wPalF.g01.lstCatalgF.preferredSize.width = 210;
  wPalF.g01.lstCatalgF.addEventListener  ("keydown", KLD_F.ListSelector);

  wPalF.g01.dedication = wPalF.g01.add ("iconbutton", undefined, File (KLD_F.sScriptDir + "\\" + "Heart.png"),{style: "toolbutton"});
  wPalF.g01.dedication.helpTip = KLD_F.UItxt.wPalF.@titleTip;

// -------------------------------------------------------------------------------- Group Find What
  wPalF.g02 = wPalF.add("group", undefined, undefined);
  wPalF.g02.orientation = "row";
  wPalF.g02.alignChildren = ["left","center"];
  wPalF.g02.spacing = 10;
  wPalF.g02.margins = 0;

  wPalF.g02.t1 = wPalF.g02.add("statictext", undefined, KLD_F.UItxt.wPalF.@g02t1);
  wPalF.g02.t1.preferredSize.width = 100;

  wPalF.g02.eFindWhat = wPalF.g02.add("edittext", undefined, undefined, { multiline: true});
  wPalF.g02.eFindWhat.graphics.font = "Consolas:14";
  wPalF.g02.eFindWhat.text = "";
  wPalF.g02.eFindWhat.preferredSize = [460, 50];
  wPalF.g02.eFindWhat.addEventListener  ("keydown", KLD_F.BtnFindByEnter);

// -------------------------------------------------- Group Options has 5 childeren, rb's are 1,2,3
  wPalF.g03 = wPalF.add("group", undefined, undefined);
  wPalF.g03.orientation = "row";
  wPalF.g03.alignChildren = ["left","center"];
  wPalF.g03.spacing = 10;
  wPalF.g03.margins = 0;

  wPalF.g03.t1 = wPalF.g03.add("statictext", undefined, KLD_F.UItxt.wPalF.@g03t1);
  wPalF.g03.t1.preferredSize.width = 100;

    wPalF.g03.rbSimple = wPalF.g03.add("radiobutton", undefined, KLD_F.UItxt.wPalF.@g03rbSimple);
    wPalF.g03.rbSimple.value = true;
    wPalF.g03.rbSimple.preferredSize.width = widthA;
    wPalF.g03.rbWild = wPalF.g03.add("radiobutton", undefined, KLD_F.UItxt.wPalF.@g03rbWild);
    wPalF.g03.rbWild.preferredSize.width = widthA;
    wPalF.g03.rbRegEx = wPalF.g03.add("radiobutton", undefined, KLD_F.UItxt.wPalF.@g03rbRegEx);
    wPalF.g03.rbRegEx.preferredSize.width = widthA;

    wPalF.g03.btnRegex = wPalF.g03.add("button", undefined, KLD_F.UItxt.wPalF.@g03btnRegex);
    wPalF.g03.btnRegex.helpTip = KLD_F.UItxt.wPalF.@g03btnRegexhelpTip;
    wPalF.g03.btnRegex.preferredSize.width = widthA;
    wPalF.g03.btnRegex.enabled = false;

  wPalF.g04 = wPalF.add("group", undefined, undefined);
  wPalF.g04.orientation = "row";
  wPalF.g04.alignChildren = ["left","center"];
  wPalF.g04.spacing = 10;
  wPalF.g04.margins = 0;

  wPalF.g04.t1 = wPalF.g04.add("statictext", undefined, ""); // space only
  wPalF.g04.t1.preferredSize.width = 100;

  wPalF.g04.chkCase = wPalF.g04.add("checkbox", undefined, KLD_F.UItxt.wPalF.@g04chkCase);
  wPalF.g04.chkCase.preferredSize.width = widthA;

  wPalF.g04.chkWord = wPalF.g04.add("checkbox", undefined, KLD_F.UItxt.wPalF.@g04chkWord);
  wPalF.g04.chkWord.preferredSize.width = widthA;

  wPalF.g04.chkBack = wPalF.g04.add("checkbox", undefined, KLD_F.UItxt.wPalF.@g04chkBack);
  wPalF.g04.chkBack.preferredSize.width = widthA;

  wPalF.g04.chkClone = wPalF.g04.add("checkbox", undefined, KLD_F.UItxt.wPalF.@g04chkClone);
  wPalF.g04.chkClone.preferredSize.width = widthA;

// ----------------------------------------------------------------------------- Group Replace Mode
  wPalF.g05 = wPalF.add("group", undefined, undefined);
  wPalF.g05.orientation = "row";
  wPalF.g05.alignChildren = ["left","center"];
  wPalF.g05.spacing = 10;
  wPalF.g05.margins = 0;

  wPalF.g05.t1 = wPalF.g05.add("statictext", undefined, KLD_F.UItxt.wPalF.@g05t1);
  wPalF.g05.t1.preferredSize.width = 100;

  wPalF.g05.rbText = wPalF.g05.add("radiobutton", undefined, KLD_F.UItxt.wPalF.@g05rbText);
  wPalF.g05.rbText.value = true;
  wPalF.g05.rbText.preferredSize.width = widthA;
  wPalF.g05.rbPaste = wPalF.g05.add("radiobutton", undefined, KLD_F.UItxt.wPalF.@g05rbPaste);
  wPalF.g05.rbPaste.preferredSize.width = widthA;
  wPalF.g05.rbPasteNoFmt = wPalF.g05.add("radiobutton", undefined, KLD_F.UItxt.wPalF.@g05rbPasteNoFmt);
  wPalF.g05.rbPasteNoFmt.preferredSize.width = widthA;
  wPalF.g05.rbSpecials = wPalF.g05.add("radiobutton", undefined, KLD_F.UItxt.wPalF.@g05rbSpecials);
  wPalF.g05.rbSpecials.preferredSize.width = 125;

// ------------------------------------------------------------------------------- Group Replace by
  wPalF.g06 = wPalF.add("group", undefined, undefined);
  wPalF.g06.orientation = "row";
  wPalF.g06.alignChildren = ["left","top"];
  wPalF.g06.spacing = 10;
  wPalF.g06.margins = 0;

  wPalF.g06.t1 = wPalF.g06.add("statictext", undefined, KLD_F.UItxt.wPalF.@g06t1);
  wPalF.g06.t1.preferredSize.width = 100;

  wPalF.g06.eReplaceBy = wPalF.g06.add("edittext", undefined, undefined, {multiline: true});
  wPalF.g06.eReplaceBy.graphics.font = "Consolas:14";
  wPalF.g06.eReplaceBy.text = "";
  wPalF.g06.eReplaceBy.preferredSize = [460, 50];

  wPalF.g06a = wPalF.add("group", undefined, undefined);
  wPalF.g06a.orientation = "row";
  wPalF.g06a.alignChildren = ["left","top"];
  wPalF.g06a.spacing = 10;
  wPalF.g06a.margins = 0;
  wPalF.g06a.t1 = wPalF.g06a.add("statictext", undefined, undefined); // space only
  wPalF.g06a.t1.preferredSize.width = 100;
  wPalF.g06a.t2 = wPalF.g06a.add("statictext", undefined, KLD_F.UItxt.wPalF.@g06at1);
  wPalF.g06a.t2.preferredSize.width = 210;
  wPalF.g06a.lstCatalgF = wPalF.g06a.add("dropdownlist", undefined, KLD_F.asCatalogF);
  wPalF.g06a.lstCatalgF.graphics.font = "dialog:12";
  wPalF.g06a.lstCatalgF.selection = -1;            // Fix for second to nth invocation
  wPalF.g06a.lstCatalgF.preferredSize.width = 240;
  wPalF.g06a.lstCatalgF.addEventListener ("keydown", KLD_F.ListSelector);

// ------------------------------------------------------------------------------------ Group Scope
  wPalF.g07 = wPalF.add("group", undefined, undefined);
  wPalF.g07.orientation = "row";
  wPalF.g07.alignChildren = ["left","center"];
  wPalF.g07.spacing = 10;
  wPalF.g07.margins = 0;

  wPalF.g07.t1 = wPalF.g07.add("statictext", undefined, KLD_F.UItxt.wPalF.@g07t1);
  wPalF.g07.t1.preferredSize.width = 100;

  wPalF.g07.rbBook = wPalF.g07.add("radiobutton", undefined, KLD_F.UItxt.wPalF.@g07rbBook);
  wPalF.g07.rbBook.preferredSize.width = 100;
  wPalF.g07.rbDoc = wPalF.g07.add("radiobutton", undefined, KLD_F.UItxt.wPalF.@g07rbDoc);
  wPalF.g07.rbDoc.preferredSize.width = 100;
  wPalF.g07.rbSelect = wPalF.g07.add("radiobutton", undefined, KLD_F.UItxt.wPalF.@g07rbSelect);
  wPalF.g07.rbSelect.preferredSize.width = 100;

// --------------------------------------------------------------------------- Group action buttons
  wPalF.g08 = wPalF.add("group", undefined, undefined);
  wPalF.g08.orientation = "row";
  wPalF.g08.alignChildren = ["left","center"];
  wPalF.g08.spacing = 10;
  wPalF.g08.margins = 0;

  wPalF.g08.btnFind = wPalF.g08.add("button", undefined, KLD_F.UItxt.wPalF.@g08btnFind);
  wPalF.g08.btnFind.preferredSize.width = 135;
  wPalF.g08.btnFind.preferredSize.height = 30;

  wPalF.g08.btnReplace = wPalF.g08.add("button", undefined, KLD_F.UItxt.wPalF.@g08btnReplace);
  wPalF.g08.btnReplace.preferredSize.width = 135;
  wPalF.g08.btnReplace.preferredSize.height = 30;

  wPalF.g08.btnReplFnd = wPalF.g08.add("button", undefined, KLD_F.UItxt.wPalF.@g08btnReplFnd);
  wPalF.g08.btnReplFnd.preferredSize.width = 135;
  wPalF.g08.btnReplFnd.preferredSize.height = 30;

  wPalF.g08.btnReplAll = wPalF.g08.add("button", undefined, KLD_F.UItxt.wPalF.@g08btnReplAll);
  wPalF.g08.btnReplAll.preferredSize.width = 135;
  wPalF.g08.btnReplAll.preferredSize.height = 30;

// ---------------------------------------------------------------------------- Group save/retrieve
  wPalF.g09 = wPalF.add("group", undefined, undefined);
  wPalF.g09.orientation = "row";
  wPalF.g09.alignChildren = ["left","center"];
  wPalF.g09.spacing = 10;
  wPalF.g09.margins = 0;

  wPalF.g09.btnSave = wPalF.g09.add("button", undefined, KLD_F.UItxt.wPalF.@g09btnSave);
  wPalF.g09.btnSave.preferredSize.width = 100;

  wPalF.g09.lstSettingName = wPalF.g09.add("dropdownlist", undefined);
  wPalF.g09.lstSettingName.selection = 0;         // start with empty entry
  wPalF.g09.lstSettingName.preferredSize.width = 240;
  wPalF.g09.lstSettingName.addEventListener ("keydown", KLD_F.ListSelector);
  wPalF.g09.lstSettingName.evParm = KLD_F.asSettingName;

  wPalF.g09.btnRetrieve = wPalF.g09.add("button", undefined, KLD_F.UItxt.wPalF.@g09btnRetrieve);
  wPalF.g09.btnRetrieve.preferredSize.width = 100;

  wPalF.g09.btnDel = wPalF.g09.add("button", undefined, KLD_F.UItxt.wPalF.@g09btnDel);
  wPalF.g09.btnDel.preferredSize.width = 100;

// ------------------------------------------------------------ Group general buttons and info area
  wPalF.g10 = wPalF.add("group", undefined, undefined);
  wPalF.g10.orientation = "row";
  wPalF.g10.alignChildren = ["left","center"];
    wPalF.g10.g01 = wPalF.g10.add ("group", undefined, undefined);
    wPalF.g10.g01.orientation = "column";
    wPalF.g10.g01.btnSaveNew = wPalF.g10.g01.add("button", undefined, KLD_F.UItxt.wPalF.@g10g01btnSaveNew);
    wPalF.g10.g01.btnSaveNew.preferredSize.width = 100;

    wPalF.g10.g01.btnHelp = wPalF.g10.g01.add("button", undefined, KLD_F.UItxt.wPalF.@g10g01btnHelp);
    wPalF.g10.g01.btnHelp.helpTip = KLD_F.UItxt.wPalF.@g10g01btnHelphelpTip;
    wPalF.g10.g01.btnHelp.preferredSize.width = 100;

    wPalF.g10.eInfo = wPalF.g10.add("edittext", undefined, undefined, {multiline: true});
    wPalF.g10.eInfo.graphics.font = "Consolas:12";
    wPalF.g10.eInfo.preferredSize.width = 350;
    wPalF.g10.eInfo.preferredSize.height = 60;

    wPalF.g10.g02 = wPalF.g10.add ("group", undefined, undefined);
    wPalF.g10.g02.orientation = "column";
    wPalF.g10.g02.btnRestore = wPalF.g10.g02.add("button", undefined, KLD_F.UItxt.wPalF.@g10g02btnRestore);
    wPalF.g10.g02.btnRestore.helpTip = KLD_F.UItxt.wPalF.@g10g02btnRestoreTip;
    wPalF.g10.g02.btnRestore.preferredSize.width = 100;

    wPalF.g10.g02.btnDone = wPalF.g10.g02.add("button", undefined, KLD_F.UItxt.wPalF.@g10g02btnDone);
    wPalF.g10.g02.btnDone.preferredSize.width = 100;

// ############################################################### Event-handler Callback functions
// There must not be parameters for externalised callback functions!
// ... if the functions are objects, then the call must not contain () !
// ----------------------------------------------------------------------- behaviour of edit fields
// There are no onClick and onDoubleClick for edit fields - Event Listeners not possible either.
// onChanging           immediate feed back during chanigng
// onChange             feedback after focus change, e.g. by TAB key
  wPalF.g02.eFindWhat.onChanging = function () {
    wPalF.g08.btnFind.enabled  = true;            // active allows only 1 character typed
    KLD_F.bFirstFind = true;                      // Find cycle starts
    KLD_F.ClearInfoArea ();
  };
  wPalF.g06.eReplaceBy.onChanging = function () {
    KLD_F.SetReplButtons (true);
    KLD_F.ClearInfoArea ();
  };
// ------------------------------------------------------------------------- behaviour of the lists
//----------------------------------------------- Find category
  wPalF.g01.lstFindTypeF.onActivate = function () {
    KLD_F.evBuffer = "";
  }
  wPalF.g01.lstFindTypeF.onChange = function () {
    KLD_F.SelectFindType ();
  }
//----------------------------------------------- Item in type catalogue
  wPalF.g01.lstCatalgF.onActivate   = function () {
    wPalF.g01.lstCatalgF.evParm = KLD_F.asCatalogF;
    KLD_F.evBuffer = "";
  }
  wPalF.g01.lstCatalgF.onChange     = function () {
    KLD_F.SelectedInCatalogA ();
  }
//----------------------------------------------- Item in type catalogue
  wPalF.g06a.lstCatalgF.onActivate   = function () {
    wPalF.g06a.lstCatalgF.evParm = KLD_F.asCatalogF;
    KLD_F.evBuffer = "";
  }
  wPalF.g06a.lstCatalgF.onChange     = function () {
    KLD_F.SelectedInCatalogB ();
  }
//----------------------------------------------- Item in Settings list
  wPalF.g09.lstSettingName.onActivate   = function () {
    wPalF.g09.lstSettingName.evParm = KLD_F.asSettingName;
    KLD_F.evBuffer = "";
  }
  wPalF.g09.lstSettingName.onChange     = function () {
    KLD_F.DisplaySettingsInfo ();
  }
// ------------------------------------------------------------ behaviour of check boxes and radios
  wPalF.g03.rbSimple.onClick = KLD_F.SelectSearchMode;
  wPalF.g03.rbWild.onClick   = KLD_F.SelectSearchMode;
  wPalF.g03.rbRegEx.onClick  = KLD_F.SelectSearchMode;

  wPalF.g04.chkWord.onClick       = function () {
    KLD_F.ClearInfoArea ();
    KLD_F.bFirstFind = true;                      // Find cycle starts
  }
  wPalF.g04.chkCase.onClick       = function () {
    KLD_F.ClearInfoArea ();
    KLD_F.bFirstFind = true;                      // Find cycle starts
  }
  wPalF.g04.chkBack.onClick       = function () {
    KLD_F.ClearInfoArea ();
    KLD_F.bFirstFind = true;                      // Find cycle starts
    if (wPalF.g04.chkBack.value) {
      KLD_F.WarnBackwardsSearch ();
    }
  }

  wPalF.g05.rbText.onClick        = function () {KLD_F.ClearInfoArea ();}
  wPalF.g05.rbPaste.onClick       = function () {KLD_F.ClearInfoArea ();}
  wPalF.g05.rbPasteNoFmt.onClick  = function () {KLD_F.ClearInfoArea ();}
  wPalF.g05.rbSpecials.onClick    = function () {
    var indxTbl;
    KLD_F.SetReplButtons (false);                 // switch on by change in Replace by
    indxTbl = wPalF.g01.lstFindTypeF.selection.index;
    if (KLD_F.asFType[indxTbl] != "USRSTR" && KLD_F.asFType[indxTbl] != "PGFLCK") {
      KLD_F.GetCatalogAny ();
      wPalF.g06a.lstCatalgF.enabled = true;
    }
  }

  wPalF.g07.rbBook.onClick   = function () {
    KLD_F.PrepareBookProc ();
  }
  wPalF.g07.rbDoc.onClick    = function () {KLD_F.ClearInfoArea ();}
  wPalF.g07.rbSelect.onClick = function () {KLD_F.ClearInfoArea ();}
// ----------------------------------------------------------------------- behaviour of the buttons
  wPalF.g03.btnRegex.onClick        =   KLD_F.ButtonRegex;
  wPalF.g08.btnFind.onClick         =   KLD_F.ButtonFind;
  wPalF.g08.btnReplace.onClick      =   KLD_F.ButtonReplace;
  wPalF.g08.btnReplFnd.onClick      =   KLD_F.ButtonReplFnd;
  wPalF.g08.btnReplAll.onClick      =   KLD_F.ButtonReplAll;
  wPalF.g09.btnSave.onClick         =   function () {
    KLD_F.ButtonSave (false);
  }
  wPalF.g09.btnRetrieve.onClick     =   function () {
    KLD_F.ButtonRetrieve (KLD_F.iSettings);       // item index in settings XML file
  }
  wPalF.g09.btnDel.onClick          =   KLD_F.ButtonDelete;
  wPalF.g10.g01.btnSaveNew.onClick  =   KLD_F.ButtonSaveNew;
  wPalF.g10.g02.btnRestore.onClick  =   KLD_F.ButtonRestore;

  wPalF.g10.g01.btnHelp.onClick    = function () {
    KLD_F.ClearInfoArea ();
    KLD_F.OpenHelpFile();
  };

  wPalF.g10.g02.btnDone.onClick    = function () {
    KLD_F.ButtonSave (true);                      // save current settings
    wPalF.close();                                // re-invoking palette is not OK
  };
// ---------------------------------------------------------------------------- Final work in panel
  KLD_F.GetColourNames ();                        // get the localised System colour names
  KLD_F.FillFindTypeData ();                      // fill FindType arrays with values from XML file
  KLD_F.FillDlgList (wPalF.g01.lstFindTypeF, KLD_F.asFindType, false); // find type
  wPalF.g01.lstFindTypeF.evParm = KLD_F.asFindType;
  wPalF.g01.lstFindTypeF.selection = KLD_F.iFindDefault;               // Text

  KLD_F.GetSettingsList ();                       // fill drop down list "saved settings"
  KLD_F.BackupSettings ();                        // preserve current state of settings

  wPalF.g07.rbBook.value  = KLD_F.bBookActive ? true : false;
  wPalF.g07.rbDoc.value   = KLD_F.bDocActive  ? true : false;
  KLD_F.DisplayInfoF (KLD_F.UItxt.General.@Warning, "red");
  if (KLD_F.bBookActive) {
    KLD_F.GetBookComponents();
    KLD_F.oCurrentComp = KLD_F.aoBookComps[0].Comp;
  }
//
  if (KLD_F.GetTextOfSelection() != "") {         // Initial state
    wPalF.g02.eFindWhat.text   = KLD_F.GetTextOfSelection();
    wPalF.g08.btnFind.active   = true;
  } else {
    wPalF.g02.eFindWhat.active = true;
  }
  wPalF.show();
} //--- end of PalFindRepl <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

KLD_F.ButtonDelete = function () { // =============================================================
/*            Delete the selected item in the XML structure - and update the file
Returns       -
Called by     PalFindRepl
Calling       DisplayInfoF, WriteXMLdata, ConvertSelfClosingElements, GetSettingsList, ClearInfoArea
History       2020-12-17
*/
var jItem, sXmlFile;

  sXmlFile = KLD_F.APP_NAME + ".xml";
  jItem = KLD_F.iSettings;
  if (jItem == undefined) { return;  }            // nothing selected
  if (jItem == 0) {
    KLD_F.DisplayInfoF (KLD_F.UItxt.ButtonDelete.@msg_01, "red"); // Do not delete the first item
    return;
  }
  delete KLD_F.xSettings.item[jItem];
  if (KLD_F.WriteXMLdata (KLD_F.xSettings, sXmlFile)) { // Recreate the file from XML object
    KLD_F.ConvertSelfClosingElements (sXmlFile);
    KLD_F.GetSettingsList ();                     // Recreate drop down settings list
//  KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonDelete.@msg_02, sName), "black");
    KLD_F.ClearInfoArea();                        // alternative to msg_02
  } else {
    KLD_F.DisplayInfoF (KLD_F.Message ("e", KLD_F.UItxt.ButtonDelete.@msg_03, sName), "red");
  }
} //--- end of ButtonDelete -----------------------------------------------------------------------

KLD_F.ButtonFind = function () { // ===============================================================
/*            One of the main functions of this script suite - find something
Called by     ButtonReplFnd, ButtonReplAll
Calling       OpenDocument, DisplayInfoF, ClearInfoArea, IsGroupableObject, UnGroup, IsSelection,
              SetReplButtons, IsOneOf, FindInSelection, ExecuteFCodes, FindSomething, IsWrapped,
              GetSystemColourIndex, IsGraphicObject, GetNextObjectInDoc, FindColourUse, FindEmptyTextLine,
              FindOverflowCell, FindOverflowText, FindUserString
Attention     For some found objects the selection can not be visualised.
History       2020-12-05
*/
var aMarkers, aMatch, aVars, bBack, bCase, bInSelection, bFinito, bNotFound, bWord, iFT, iMode, index, j, kRsv,
    oDoc, oFound= {}, oMarker, oParent, oRow, aTbls, oTbl, oTextItem, oTL1, oTL2, oTR, oTLstart, oType, oVar,
    sDefsName, sFound, sFType, sSearch, sText, sType, sUstring, aXrefs, oXref;

  if (KLD_F.bBookActive) {                        // --- book
    if (KLD_F.bNextCompOK) {
      KLD_F.kCurrentComp ++;
      if (KLD_F.kCurrentComp < KLD_F.aoBookComps.length) { // close previous doc ?
        KLD_F.oCurrentComp = KLD_F.aoBookComps[KLD_F.kCurrentComp].Comp;
        KLD_F.bNextCompOK = false;
        oDoc = KLD_F.OpenDocument (KLD_F.oCurrentComp.Name, true); // Get no insertion error for invisible!
        KLD_F.oCurrentDoc = oDoc;
        KLD_F.bFirstFind = true;                  // Find cycle starts
      } else {
        KLD_F.DisplayInfoF (KLD_F.UItxt.ButtonFind.@msg_15, "red");
        return;
      }
    }
  } else {                                        // --- ordinary document
 // KLD_F.bFirstFind = true;  --- not to be done here - would be set on each Find.
  }
  oDoc = KLD_F.oCurrentDoc;                       // stand-alone, second-to last item in component
  oTLstart = new TextLoc (oDoc.FirstPgfInDoc, 0);
  KLD_F.ClearInfoArea ();
  oTR = oDoc.TextSelection; // <<<<
  if (KLD_F.oLstObjFound != undefined ) {
    oFound = KLD_F.oLstObjFound.object;
  }
  if (oFound != undefined && oFound.constructor.name == "Cell") { // escaping into TR
    oTL1 = new TextLoc (oFound.FirstPgf, 0);
    oTR  = new TextRange (oTL1, oTL1);
    oDoc.TextSelection = oTR;
  }
  if (oFound != undefined) {                      // try to escape
    if (KLD_F.IsGroupableObject (oFound)  && (oFound.GroupParent.BorderWidth > 0)) {
//  Borderwidth: arbitrary property to indicate validity of Group - no other criterion found
      if (oFound.constructor.name != "TextLine") {
        KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonFind.@msg_11, oFound.constructor.name), "red");
        KLD_F.UnGroup (oFound.GroupParent);
        KLD_F.oLstObjFound.object = undefined;
        oDoc.TextSelection = new TextRange (oTLstart, oTLstart);
        return;
      }
    } else if (oFound.constructor.name == "AFrame"){
      oTL1 = oFound.TextLoc;
      oTR  = new TextRange (oTL1, oTL1);
      oDoc.TextSelection = oTR;
    }
  }
  if (!KLD_F.IsSelection (oDoc)) {                // No selection/insert point
    alert (KLD_F.UItxt.ButtonFind.@msg_03.toString());
    KLD_F.DisplayInfoF (KLD_F.UItxt.ButtonFind.@msg_03.toString(), "red");
    return;
  }
  KLD_F.SetReplButtons (false);
  iFT       = wPalF.g01.lstFindTypeF.selection.index;
  sFType    = KLD_F.asFType[iFT]
  sSearch   = wPalF.g02.eFindWhat.text;
  bWord     = wPalF.g04.chkWord.value;
  bCase     = wPalF.g04.chkCase.value;
  bBack     = wPalF.g04.chkBack.value;
  bRegEx    = wPalF.g03.rbRegEx.value;
  if (wPalF.g03.rbSimple.value)    {iMode = 0;}
  else if (wPalF.g03.rbWild.value) {iMode = 1;}
  else                             {iMode = 2;}
  bInSelection = wPalF.g07.rbSelect.value;

// ============================================== Standard Find types =============================
  if (KLD_F.IsOneOf ("ANCFRM AUTHYP CHRFMT CHRFMO CHRTAG CONTAG FOONOT MRKANY MRKTXT MRKTYP OSTFMO OSTTAG PGFFMO PGFTAG RUBANY TBLFMO TBLANY TBLTAG TXTCLP TXTTXT TINANY TINUNR VARANY VARNAM XRFANY XRFFMT XRFUNR ", sFType) > 0) {
    if (bInSelection) {
      oTR = KLD_F.FindInSelection (sFType, sSearch, iMode, bWord, bCase, false); // back not working
      oDoc.TextSelection = oTR;
      if (sFType == "AUTHYP") {
        KLD_F.ExecuteFCodes(["SelectPreviousCharacter"]); // indicate selection
      }
      oFound = oTR;
    } else {
      //if (KLD_F.IsOneOf ("AUTHYP TXTTXT ", sFType) > 0) {
        //oTL1 = new TextLoc (oTR.beg.obj, oTR.beg.offset+1); // avoid finding the same only
        //oDoc.TextSelection = new TextRange (oTL1, oTL1);
      //}
      oTR = KLD_F.FindSomething (sFType, sSearch, iMode, bWord, bCase, bBack);
      if (KLD_F.IsWrapped (oTR)) { KLD_F.iError = -95;} // Force "Not found anymore"
      if (KLD_F.iError != -95) {
        if (KLD_F.IsOneOf ("AFrame Cell Fn Pgf SubCol ", oTR.beg.obj.constructor.name) > 0) { // can have InTextObj
          if (oTR.beg.obj.InTextObj.Overflowed === 1) { // attribute Overflowed in: Cell, Fn, Subcol and Tbl
            KLD_F.DisplayInfoF (KLD_F.UItxt.ButtonFind.@msg_05.toString(), "red"); // probably hidden in overflow
          }
        }
        if (oTR.beg.obj.ObjectValid()) {              // e.g. Pgf
          oFound = oTR.beg.obj;                       // ??? analogous to non-standard Find Types
  //                  ----------------------------- Provide information from found object(s) --------
          if        (sFType == "MRKANY" || sFType == "MRKTXT" || sFType == "MRKTYP") { // ---------- Any |  of  Text | of Type
            aMarkers  = oDoc.GetTextForRange (oTR, Constants.FTI_MarkerAnchor); // should be only one
            oMarker   = aMarkers[0].obj;
            KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonFind.@msg_06, oMarker.MarkerTypeId.Name, oMarker.MarkerText), "black")

          } else if (sFType == "TBLANY" || sFType == "TBLTAG") { // ---------- Any table | Table of format
            aTbls  = oDoc.GetTextForRange (oTR, Constants.FTI_TblAnchor);
            oTbl   = aTbls[0].obj;                                            //    name
            KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonFind.@msg_10, oTbl.TblTag), "black")

          } else if (sFType == "VARANY" || sFType == "VARNAM") { // ---------- Any variable | named variable
            aVars  = oDoc.GetTextForRange (oTR, Constants.FTI_VarBegin);
            oVar   = aVars[0].obj;                                            //    name              content
            KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonFind.@msg_08, oVar.VarFmt.Name, oVar.VarFmt.Fmt), "black")

          } else if (sFType == "XRFANY" || sFType == "XRFFMT") { // ---------- Any Xref | Xref of format
            aXrefs  = oDoc.GetTextForRange (oTR, Constants.FTI_XRefBegin);
            oXref   = aXrefs[0].obj;                                            //    name
            KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonFind.@msg_09, oXref.XRefFmt.Name), "black")
          } else if (sFType == "OSTFMO")                       { // ---------- Object style override

          }
        }
//      if (KLD_F.IsOneOf ("OSTFMO TBLFMO", sFType) > 0) { // --- prepare for next search
//        oParent = oTR.beg.obj.FrameParent;
//        KLD_F.AdvanceTextLoc  (oParent);
//      }
      }
    }
  } else { // =================================== Non=standard Find types =========================
    if (KLD_F.oLstObjFound == undefined) {        // 2022-05-03 11:11
        KLD_F.oLstObjFound = {};
    }
  	if        (sFType == "COLOUR") {              // ---------- Colour ----------------------------
      kRsv = KLD_F.GetSystemColourIndex(sSearch);
      if (KLD_F.oLstObjFound.object == undefined) { // inital find
        oFound    = oDoc.FirstGraphicInDoc;
        KLD_F.oLstObjFound.object = oFound;
      } else {
        oFound = KLD_F.oLstObjFound.object;
        if (oFound != undefined && KLD_F.IsGraphicObject(oFound)) {
          KLD_F.oLstObjFound.object.GraphicIsSelected = false; // works on graphic objects only
        }
        oFound = KLD_F.GetNextObjectInDoc(oFound);// continue search
        KLD_F.oLstObjFound.object = oFound;
      }
      oFound = KLD_F.FindColourUse (kRsv, sSearch);
      if (oFound != undefined) {                  // a relevant object found
        if (KLD_F.IsGraphicObject(oFound)) {
          if (oFound.Pen == 15 && oFound.Fill == 15) { // 15: FV_FILL_CLEAR
            KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonFind.@msg_04, sSearch, oFound.constructor.name), "red");
          }
        }
        KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonFind.@msg_14, sSearch, oFound.constructor.name), "red");
      }
      KLD_F.oLstObjFound.object = oFound;

  	} else if (sFType == "EMTTXL") {              // ---------- Empty TextLine --------------------
      if (typeof KLD_F.oLstObjFound.object == undefined || KLD_F.oLstObjFound.object == undefined) { // inital find
        KLD_F.oLstObjFound.object = oDoc.FirstGraphicInDoc;
      } else {
        KLD_F.oLstObjFound.object.GraphicIsSelected = false;           // unselect previous find
        KLD_F.oLstObjFound.object = KLD_F.oLstObjFound.object.NextGraphicInDoc;
      }
      oFound = KLD_F.FindEmptyTextLine(KLD_F.oLstObjFound);
      if (oFound != undefined) {                  // empty TextLine found
        KLD_F.SetReplButtons (true);
        return;
      }

    } else if (sFType == "OVLCEL") {              // ---------- Overflowing cell ------------------
      oFound = KLD_F.oLstObjFound.object;
      if (oFound == undefined || oFound.constructor.name != "Tbl") { // inital find
        oTbl = oDoc.FirstTblInDoc;
        KLD_F.oLstObjFound.object  = oTbl;
        KLD_F.oLstObjFound.row  = oTbl.FirstRowInTbl;
        KLD_F.oLstObjFound.cell = KLD_F.oLstObjFound.row.FirstCellInRow;
      }
      oFound = KLD_F.FindOverflowCell (KLD_F.oLstObjFound);
      if (oFound != undefined) {
        return;                                   // Find only
      }

  	} else if (sFType == "OVLTXT") {              // ---------- Overflowing text flow -------------
      oFound = KLD_F.oLstObjFound.object;
      if (oFound == undefined || oFound.constructor.name != "Flow") { // inital find
        oFlow = oDoc.FirstFlowInDoc;
        KLD_F.oLstObjFound.object = oFlow;
      }
      oFound = KLD_F.FindOverflowText (KLD_F.oLstObjFound);
      if (oFound != undefined) {
        KLD_F.oLstObjFound.object = KLD_F.oLstObjFound.object.NextFlowInDoc;
        return;                                   // Find only
      }

    } else if (sFType == "PGFLCK") {              // ---------- Locked Pgf ------------------------
      oFound = KLD_F.oLstObjFound.object;
      oFound = KLD_F.FindLockedParagraph (oFound); // contains test for Text Inset
      if (oFound != undefined) {
        KLD_F.DisplayObject(oFound.object); // I want to see it
        KLD_F.oLstObjFound.object = KLD_F.oLstObjFound.object.NextPgfInFlow;
        KLD_F.SetReplButtons (true);
        return;
      }

    } else if (sFType == "USRSTR") {              // ---------- User String -----------------------
      if (KLD_F.jCurrentUStr < KLD_F.nUserStrings) {
        if (sSearch == "") {
          sSearch = undefined;
        }
        oFound = KLD_F.FindUserString (oDoc, KLD_F.aoUserStrings, sSearch, bCase, bWord, bRegEx);
        if (oFound !== undefined) {
          oFound = KLD_F.aoUserStrings[KLD_F.jCurrentUStr].ObjObj;
          sType  = oFound.constructor.name;
          sUstring = KLD_F.aoUserStrings[KLD_F.jCurrentUStr].Ustring;
          KLD_F.DisplayInfoF (KLD_F.Message("i", KLD_F.UItxt.ButtonFind.@msg_13, sType, sUstring), "black");
          oTL1 = KLD_F.aoUserStrings[KLD_F.jCurrentUStr].TL;
          oTL2 = new TextLoc (oTL1.obj, oTL1.offset +1);
          oDoc.TextSelection = new TextRange (oTL1, oTL2);
          oDoc.ScrollToText (oDoc.TextSelection);
          KLD_F.jCurrentUStr += 1;                // for next search
          KLD_F.oLstObjFound.object = oFound;
        }
      } else {
        oFound = undefined;
      }
  	}
  }
  if (oFound === undefined || (KLD_F.iError != 0)) {  // ----------------- Nothing found -----------
    KLD_F.DisplayInfoF (KLD_F.UItxt.ButtonFind.@msg_07.toString(), "red"); // not found (anymore)
    KLD_F.bNextCompOK = true;                     // propably look into next book component
  } else {                   // ----------------- Find success, Find next is possible -------------
    if (KLD_F.IsOneOf ("TINUNR XRFUNR ", sFType) <= 0) { // Repl buttons left disabled for Unresolveds
      KLD_F.SetReplButtons (true);
    }
  }
} //--- end of ButtonFind -------------------------------------------------------------------------

KLD_F.ButtonRegex = function () { // ==============================================================
/*            Provide ancillary information about RegEx - ulitmately allow to insert such stuff
Called by     PalFindRepl
Calling       MakerIniRead, DisplayInfoF, PalRegEx
History       2020-12-05
*/
var sFlavour, sURL;
  sFlavour = KLD_F.MakerIniRead ("Preferences", "RegularExpressionSyntax");
  if (sFlavour != "Perl") {
    KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonRegex.@msg_01, sFlavour), "red");
    wPalF.show();
//  $.sleep (5000);
  }
  KLD_F.PalRegEx();                               // open ancillary palette
  wPalR.g3.eReg.text = wPalF.g02.eFindWhat.text;  // use current RegEx as a start
} //--- end of ButtonRegex ------------------------------------------------------------------------

KLD_F.ButtonReplace = function () { // ============================================================
/*            Replace function
Called by     ButtonReplFnd, ButtonReplAll
Calling       ClearInfoArea, GetText, Message, ReplaceInString, PreserveSpace, GetTextOfSelection,
              GetClonedCase, UnformatClipboard, IsOneOf, RemoveOverride, ExecuteFCodes, ApplyCharFmt,
              RemoveCondFmtFromRow, ApplyCondFmtToRow, RemoveConditionsFromText, ApplyCondFmtToText,
              ApplyObjectStyle, ApplyPgfFmt, ApplyTblFmt, DisplayInfoF, SetReplButtons
History       2022-04-22
*/
var aTextItems, bCase, bClone, bWord, bRegEx, bRow, iFT, iRT, iTxtLen, j, jOffset, oColour,
    oObject, oDoc, oGroup, oTbl, oTL, oTLstart, oRow, oTbl, oTR, oVar,oMarker, oXref, oXrfFmnt,
    sDef, sName, sFType, sFound, sReplace, sResult, sRType, sSearch, sText, sXRefDoc, sXRefID ;


  oDoc     = app.ActiveDoc;
  oTLstart = new TextLoc (oDoc.FirstPgfInDoc, 0);
  KLD_F.ClearInfoArea ();
  iFT    = wPalF.g01.lstFindTypeF.selection.index;
  sFType = KLD_F.asFType[iFT];                    // what object do we handle ?
  bWord     = wPalF.g04.chkWord.value  ? true : false;
  bCase     = wPalF.g04.chkCase.value  ? true : false;
  bRegEx    = wPalF.g03.rbRegEx.value  ? true : false;
  bClone    = wPalF.g04.chkClone.value ? true : false;

  oObject   = KLD_F.oLstObjFound.object;          // -- to see in data browser
  if (oObject == undefined ) {
    oObject = oDoc.FirstSelectedGraphicInDoc;
  }
// is the object one of "CHRTAG CONTAG XRFFMT MRKANY MRKTXT MRKTYP OSTTAG PGFTAG TBLTAG VARANY VARNAM"?
  if (oDoc.TextSelection.beg.obj.ObjectValid()) { // text range
    oTR      = oDoc.TextSelection;
    sText    = KLD_F.GetText(oDoc, oTR);
  } else  if (oObject != undefined ) {            // object (e.g. empty TextLine)
    oTL      = new TextLoc (oObject, 0);
    oTR      = new TextRange(oTL, oTL);
    sText    = "";
  } else {
    alert (KLD_F.Message ("e", KLD_F.UItxt.General.@ProgramError, "ButtonReplace-1"));
    return;
  }
  sSearch   = wPalF.g02.eFindWhat.text;
  sReplace  = wPalF.g06.eReplaceBy.text;
  if (sReplace == undefined) {sReplace = "";}

  if (wPalF.g05.rbText.enabled && wPalF.g05.rbText.value) { // ~~~ Replacing by text ~~~~~~~~~~~~~~
    if (sFType == "PGFTAG") {oTR.end.offset -= 1;} // do notinclude ¶ end
    oDoc.TextSelection = oTR;                     // set up the text range to clear the original text
    if (bRegEx) {
      sResult = KLD_F.ReplaceInString (sText, sSearch, sReplace, bCase, bWord, bRegEx)
      if (sResult != "") {sResult = KLD_F.PreserveSpace (oDoc, sResult);}
      oDoc.Clear(0);                              // clear current selection
      oDoc.AddText(oTR.beg, sResult);             // insert the new text at the original beginning of the text range
      oTR.beg.offset += sResult.length;           // replace the text range in memory
    } else {
      if (sReplace != "") {sReplace = KLD_F.PreserveSpace (oDoc, sReplace);}
      if (bClone) {                               // modify replacement
        sFound = KLD_F.GetTextOfSelection();
        sReplace = KLD_F.GetClonedCase (sFound, sReplace);
      }
      oDoc.Clear(0);                              // clear current selection
      oTR      = oDoc.TextSelection;              // at least necessary for replacing TI
      oDoc.AddText(oTR.beg, sReplace);            // insert the new text at the original beginning of the text range
      oTR.beg.offset += sReplace.length;          // replace the text range in memory
    }

  } else if (wPalF.g05.rbPaste.value) {           // ~~~ Replacing by Paste ~~~~~~~~~~~~~~~~~~~~~~~
    if (sFType == "PGFTAG") {oTR.end.offset -= 1;} // do notinclude ¶ end
    oDoc.TextSelection = oTR;                     // set up the text range to clear the original text
    oDoc.Paste (0);                               // 0 = not interactive - see FDK reference
    oTL = new TextLoc (oTR.end.obj, oTR.end.offset);// advance to end of inserted stuff
    oTR = new TextRange (oTL, oTL);
    oDoc.TextSelection = oTR;

  } else if (wPalF.g05.rbPasteNoFmt.value) {      // ~~~ Replacing by Paste without format ~~~~~~~~
    if (sFType == "PGFTAG") {oTR.end.offset -= 1;} // do notinclude ¶ end
    oDoc.TextSelection = oTR;                     // set up the text range to clear the original text
    KLD_F.UnformatClipboard (oDoc, oTR.beg.obj);  // change clipboard to target ¶ format
    oDoc.Paste (0);                               // 0 = not interactive - see FDK reference
    oTL = new TextLoc (oTR.end.obj, oTR.end.offset);// advance to end of inserted stuff
    oTR = new TextRange (oTL, oTL);
    oDoc.TextSelection = oTR;

  } else if (wPalF.g05.rbSpecials.value) { // ~~~ Special: Replacing by other object/property ~~~~~
    sRType = KLD_F.oReplObject.type;              // = wPalF.g06a.lstCatalgF.selection
    sName  = KLD_F.oReplObject.name;
    sDef   = KLD_F.oReplObject.def;

    if (KLD_F.IsOneOf ("CHRFMO OSTFMO PGFFMO TBLFMO ", sFType) > 0) { // --- Overrides ------------
      KLD_F.RemoveOverride (oDoc, sFType);

  	} else if (sFType == "CHRTAG" ) {             // ------- Character style ----------------------
  	  if (sName == "") {
  	      KLD_F.ExecuteFCodes (["CharacterDefaultPgfFont"]); // Remove this tag
  	  } else {
        KLD_F.ApplyCharFmt (oDoc, oTR, sName);
      }

  	} else if (sFType == "COLOUR" ) {             // ------- Colour -------------------------------
      if (sName == "") {                          // object without colour (name = null) is invisible
        oObject.Fill = 15;                        // transparent
        oObject.Pen  = 15;
        oColour = oDoc.GetNamedColor (0);         // non-forseen argument
        oObject.Color = oColour;                  // → null
      } else {
        if (oObject.Fill == 15) {oObject.Fill = 0} // make it visible
        if (oObject.Pen == 15)  {oObject.Pen = 0}
  	    oColour = oDoc.GetNamedColor (sName);
  	    oObject.Color = oColour;
      }

    } else if (sFType == "CONTAG") {              // ------- Conditional Tag ----------------------
      bRow = false, oTbl = oDoc.SelectedTbl;
      if (oTbl.ObjectValid()) {bRow = true;}
      if (bRow) {                                 // --- row in table
        oRow  = oTbl.TopRowSelection;
        if (sName == "") {
          KLD_F.RemoveCondFmtFromRow (oDoc, oRow, sSearch);
        } else {
          KLD_F.RemoveCondFmtFromRow (oDoc, oRow, sSearch);
          KLD_F.ApplyCondFmtToRow (oDoc, oRow, sReplace);
        }
      } else {                                    // --- ordinary text range
        if (sName == "") {
          KLD_F.RemoveConditionsFromText (oDoc, oTR, sSearch);
        } else {
          KLD_F.RemoveConditionsFromText (oDoc, oTR, sSearch);
          KLD_F.ApplyCondFmtToText (oDoc, oTR, sReplace); // See comments there
        }
      }

    } else if (sFType == "EMTTXL") {              // ------- Empty TextLine -----------------------
      if (oObject.GroupParent.BorderWidth > 0) {  // no other criterion found to distinguish
         oObject.GroupParent = 0;                 // ungroup this object
      }
      oObject.Delete();
      KLD_F.oLstObjFound.object = undefined;      // restart search
      oDoc.TextSelection = new TextRange (oTLstart, oTLstart);

    } else if (sFType == "MRKANY" || sFType == "MRKTYP") {// ------- Marker type ------------------
      aTextItems = oDoc.GetTextForRange(oTR, Constants.FTI_MarkerAnchor);
      if (aTextItems.len > 0) {
        oMarker = aTextItems[0].obj;
        oMarker.MarkerTypeId = oDoc.GetNamedMarkerType (sName); // was selected from catalogue
      }

    } else if (sFType == "MRKTXT") {              // Marker text ----------------------------------
      aTextItems = oDoc.GetTextForRange(oTR, Constants.FTI_MarkerAnchor);
      if (aTextItems.len > 0) {
        oMarker = aTextItems[0].obj;
      }
      if (!bRegEx) {sSearch = oMarker.MarkerText};
      sResult = KLD_F.ReplaceInString (oMarker.MarkerText, sSearch, sReplace, bCase, bWord, bRegEx)
      oMarker.MarkerText = sResult;

    } else if (sFType == "OSTTAG" ) {             // ------- Object Style Tag ---------------------
      if (!oObject.ObjectValid()) {               // not the contained object, but Frame
        aTextItems = oDoc.GetTextForRange(oTR, Constants.FTI_FrameAnchor);
        if (aTextItems.len > 0) {
          oObject = aTextItems[0].obj;
        }
      }
      if (sName == "") {
        oObject.StyleTag = undefined;             // remove style
      } else {
    	  KLD_F.ApplyObjectStyle (oDoc, oObject, sName)
    	}

    } else if (sFType == "PGFLCK" ) {             // ------- Locked Paragraph ---------------------
			oPgf = oObject.beg.obj
      oPgf.Locked = false;                        // remove the lock

    } else if (sFType == "PGFTAG" ) {             // ------- Paragraph style ----------------------
      KLD_F.ApplyPgfFmt (oDoc, oTR.beg.obj, sName);

  	} else if (sFType == "TBLANY" || sFType == "TBLTAG") { // ------- Tables ----------------------
      aTextItems = oDoc.GetTextForRange(oTR, Constants.FTI_TblAnchor);
      if (aTextItems.len > 0) {
        oTbl = aTextItems[0].obj;
      }
      KLD_F.ApplyTblFmt (oDoc, oTbl, sName);
  	} else if (sFType == "TXTTXT" ) {             // ------- Text by Variable ---------------------
      oDoc.Clear(0);
      oTL = oTR.beg;
      oVar = oDoc.NewAnchoredFormattedVar (sName, oTL);
      oTR.beg = oVar.TextRange.end;               // replace the text range in memory
  	} else if (sFType == "USRSTR" ) {             // ------- User String --------------------------
      sText = oObject.Ustring;
      sResult = KLD_F.ReplaceInString (sText, sSearch, sReplace, bCase, bWord, bRegEx)
      oObject.ObjObj.UserString = sReplace;
      oObject.Ustring = sReplace;
  	} else if (sFType == "VARANY" || sFType == "VARNAM") { // ------- Variables -------------------
      oDoc.Clear(0);
      oTL = oTR.beg;
      oVar = oDoc.GetNamedVarFmt(sName);
      oVar = oDoc.NewAnchoredFormattedVar (sName, oTL);
      oTR.beg = oVar.TextRange.end;
    } else if (sFType == "XRFANY" || sFType == "XRFFMT") { // ------- Cross References ------------
      aTextItems = oDoc.GetTextForRange(oTR, Constants.FTI_XRefBegin);
      if (aTextItems.len > 0) {
        oXref = aTextItems[0].obj;
      }
      sXRefID = oXref.XRefSrcText;
      sXRefDoc= oXref.XRefFile;
      oDoc.Clear(0);                              // clear current selection
      oTR = oDoc.TextSelection;
      oXRef = oDoc.NewAnchoredFormattedXRef(sName, oTR.beg); // Insert a new xref object
      oXRef.XRefFile    = sXRefDoc;
      oXRef.XRefSrcText = sXRefID;
      oDoc.UpdateXRef(oDoc, oXref);               // Establish the new xref.
    }

  } else {
    KLD_F.DisplayInfoF (KLD_F.UItxt.ButtonReplace.@msg_01, "red"); // no remplace mode
    return;
  }
  oTL = new TextLoc (oTR.beg.obj, oTR.beg.offset);// advance to new end for next find
  oTR = new TextRange (oTL, oTL);
  oDoc.TextSelection = oTR;
  KLD_F.SetReplButtons (false);
  KLD_F.bFirstFind = true;                        // Find cycle starts anew
  return;
} //--- end of ButtonReplace ----------------------------------------------------------------------

KLD_F.ButtonReplAll = function () { // ============================================================
/*            Replace All function
Called by     PalFindRepl
Calling       CheckForPotentialLoop, DisplayInfoF, ButtonReplace, ButtonFind
Comment       If not found any more: this button is disabled
History       2022-07-11
*/
var nDone = 0, nComponents = KLD_F.nComponents;
var bOK, bWord, sSearch, sReplace;

  bWord     = wPalF.g04.chkWord.value ? true : false;
  sSearch   = wPalF.g02.eFindWhat.text;
  sReplace  = wPalF.g06.eReplaceBy.text;
  bOK = KLD_F.CheckForPotentialLoop (sSearch, sReplace, bWord);
  if (bOK) {
    KLD_F.DisplayInfoF (KLD_F.UItxt.ButtonReplAll.@msg_02, "red");
    bOK = confirm (KLD_F.UItxt.ButtonReplAll.@msg_02.toString(), false, KLD_F.UItxt.ButtonReplAll.@msg_01.toString());
    if (bOK) {
      while (wPalF.g08.btnReplAll.enabled) {
        KLD_F.ButtonReplace ();
        nDone ++;
        KLD_F.ButtonFind ();
      }
      if (KLD_F.bBookActive) {
        KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonReplAll.@msg_04, nDone, nComponents), "red");
      } else {
        KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonReplAll.@msg_03, nDone), "red");
      }
    }
  }
} //--- end of ButtonReplAll ----------------------------------------------------------------------

KLD_F.ButtonReplFnd = function () { // ============================================================
/*            Replace and find function
Called by     PalFindRepl
Calling       CheckForPotentialLoop, ButtonReplace, ButtonFind
Comment       If not found any more: this button is disabled
History       2022-07-11
*/
var bOK, bWord, sSearch, sReplace;

  bWord     = wPalF.g04.chkWord.value ? true : false;
  sSearch   = wPalF.g02.eFindWhat.text;
  sReplace  = wPalF.g06.eReplaceBy.text;
  bOK = KLD_F.CheckForPotentialLoop (sSearch, sReplace, bWord);
  if (bOK) {
    KLD_F.ButtonReplace ();
    KLD_F.ButtonFind ();
  }
} //--- end of ButtonReplFnd ----------------------------------------------------------------------

KLD_F.ButtonRestore = function () { // ==============================================================
/*            Set the panel to it's state before last close
Called by     Notify, DisplaySettingsInfo
Calling       FindIndexFromValue, ButtonRetrieve, DisplayInfoF
History       2020-12-05
*/
var iLast, sName = KLD_F.sCloseName;

  iLast = KLD_F.FindIndexFromValue (KLD_F.asSettingName, sName); // 0 - first item in array
  KLD_F.ButtonRetrieve (iLast);                   // item index in settings XML file

  wPalF.g09.lstSettingName.selection = 0;         // show first entry - this change will set info
  wPalF.g09.btnRetrieve.enabled    = false;
  wPalF.g09.btnDel.enabled         = false;

  wPalF.g10.g01.btnHelp.enabled     = true;
  KLD_F.DisplayInfoF (KLD_F.UItxt.General.@Warning, "red");
  wPalF.g10.g02.btnDone.enabled     = true;
  wPalF.show();
} //--- end of ButtonRestore ------------------------------------------------------------------------

KLD_F.ButtonRetrieve = function (jItem) { // ======================================================
/*            Retrieve settings from XML file
Argements     jItem:          item index in settings XML file
Called by     PalFindRepl
Calling       FindIndexFromValue, SetActionButtons, SettingsToPasteBuffer
Comment       All fields in the panel are filled from the settings file
              The paste buffer is also filled (e.g. to be used for documentation)
History       2021-01-02
*/
var iSearchMode, iFT, sFindType, iPosComma, iReplMode, jItem, sFindParm, sName, xItem;

// ---------------------------------------------- Get data from the XML file
// On input toString() also converts entities (e.g. &quote;) to the string character - nice - does not work for empty elements!
  xItem = KLD_F.xSettings.item[jItem];
  sName = xItem.name.toString();
  sFindType = xItem.findtype.toString();          // "Anchored Frame"
  iFT = KLD_F.FindIndexFromValue (KLD_F.asFindType, sFindType); // 0 - first item in array
  wPalF.g01.lstFindTypeF.selection = iFT;         // sets default for searchMode, ReplMode etc.
// ---------------------------------------------- Set the dialogue to the read values
  iSearchMode = xItem.options.@searchMode.toString();
  if (iSearchMode == 0)      {wPalF.g03.rbSimple.value = true; wPalF.g03.btnRegex.enabled  = false;}
  else if (iSearchMode == 1) {wPalF.g03.rbWild.value = true; wPalF.g03.btnRegex.enabled  = false;}
  else                       {wPalF.g03.rbRegEx.value = true; wPalF.g03.btnRegex.enabled  = true;}

  wPalF.g02.eFindWhat.text  = xItem.findstring.toString();
  wPalF.g04.chkCase.value   = (xItem.options.@case.toString()  == 1) ? true : false;;
  wPalF.g04.chkWord.value   = (xItem.options.@word.toString()  == 1) ? true : false;
  wPalF.g04.chkBack.value   = (xItem.options.@back.toString()  == 1) ? true : false;
  wPalF.g04.chkClone.value  = (xItem.options.@clone.toString() == 1) ? true : false;

  iReplMode   = xItem.options.@replMode.toString();
  if (iReplMode == 0)        {wPalF.g05.rbText.value       = true;}
  else if (iReplMode == 1)   {wPalF.g05.rbPaste.value      = true;}
  else if (iReplMode == 2)   {wPalF.g05.rbPasteNoFmt.value = true;}
  else                       {wPalF.g05.rbSpecials.value   = true;}
// scope (book, doc, selection) is not saved/retrieved
  wPalF.g06.eReplaceBy.text    = xItem.replstring.toString();
  KLD_F.SetActionButtons ();                      // prophylactically set only Find
  if (sName != KLD_F.sCloseName) {
    KLD_F.SettingsToPasteBuffer ();
  }
  KLD_F.bRetrieved = true;
} //--- end of ButtonRetrieve ---------------------------------------------------------------------

KLD_F.ButtonSave = function (bClose) { // =========================================================
/*            Save the current settings; allow to add something to the description
Aarguments    bClose          true: save the current settings by name KLD_F.sCloseName
                              false: save with name given
Called by     PalFindRepl
Calling       FindIndexFromValue, Message, IsEmptyString, WriteXMLdata, ConvertSelfClosingElements,
              GetSettingsList, DisplayInfoF
Comment       It is not possible to copy text from the selcted item in the drop down.
              Hence we need two buttons and an indication how to break the info text into
              two pieces: name and info are separated by a (first) NewLine.
*/
var jItem=0, sName, sXmlFile, xItem = {};

  sXmlFile = KLD_F.APP_NAME + ".xml";
  if (bClose) {
    jItem = KLD_F.FindIndexFromValue (KLD_F.asSettingName, KLD_F.sCloseName);
  } else {
    jItem = KLD_F.iSettings;
  }
  if (jItem == undefined) {
    KLD_F.Message ("E", KLD_F.UItxt.ButtonSave.@msg_01); // advice, if empty item selected
    return;
  }
// here following checks to be added: info ≠ blank, findstring/replstring ≠ blank → KLD_F.UItxt.Genral.@ValUndefined
  xItem = KLD_F.xSettings.item[jItem];            // current item
// xItem.name                                      // this can not be changed here
  if (!bClose) {
    xItem.info      = KLD_F.IsEmptyString (wPalF.g10.eInfo.text)      ? "--- empty / leer / vide ---" : wPalF.g10.eInfo.text;
  }
  xItem.findstring  = KLD_F.IsEmptyString (wPalF.g02.eFindWhat.text)  ? "--- empty / leer / vide ---" : wPalF.g02.eFindWhat.text;
  xItem.replstring  = KLD_F.IsEmptyString (wPalF.g06.eReplaceBy.text) ? "--- empty / leer / vide ---" : wPalF.g06.eReplaceBy.text;
  xItem.findtype    = wPalF.g01.lstFindTypeF.selection;
  if (wPalF.g03.rbSimple.value)     {xItem.options.@searchMode = 0;}
  else if (wPalF.g03.rbWild.value)  {xItem.options.@searchMode = 1;}
  else                              {xItem.options.@searchMode = 2;}
  xItem.options.@case       = wPalF.g04.chkCase.value  ? 1 : 0;
  xItem.options.@word       = wPalF.g04.chkWord.value  ? 1 : 0;
  xItem.options.@back       = wPalF.g04.chkBack.value  ? 1 : 0;
  xItem.options.@clone      = wPalF.g04.chkClone.value ? 1 : 0;
  if (wPalF.g05.rbText.value)            {xItem.options.@replMode = 0;}
  else if (wPalF.g05.rbPaste.value)      {xItem.options.@replMode = 1;}
  else if (wPalF.g05.rbPasteNoFmt.value) {xItem.options.@replMode = 2;}
  else                                   {xItem.options.@replMode = 3;}
  if (bClose) {
    sName = KLD_F.sCloseName;
  } else {
    sName = xItem.name;
  }
  if (KLD_F.WriteXMLdata (KLD_F.xSettings, sXmlFile)) { // Recreate the file from XML object
    KLD_F.ConvertSelfClosingElements (sXmlFile);
    KLD_F.GetSettingsList ();                     // Recreate drop down settings list
    KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonSave.@msg_02, sName), "black");
  } else {
    KLD_F.DisplayInfoF (KLD_F.Message ("e", KLD_F.UItxt.ButtonSave.@msg_03, sName), "red");
  }
} //--- end of ButtonSave -------------------------------------------------------------------------

KLD_F.ButtonSaveNew = function () { // ============================================================
/*            Save the settings as new item
Called by     PalFindRepl
Calling       Message, IsEmptyString, WriteXMLdata, ConvertSelfClosingElements, GetSettingsList,
              DisplayInfoF
Comment       It is not possible to copy text from the selcted item in the drop down.
              Hence we need two buttons and an idication how to break the info text into two
              pieces: name and info are separated by a (first) NewLine.
*/
var jItem, jNL, NL = String.fromCharCode (10), sName, sInfo, sFindType, sFindString,
    bWord, bCase, bBack, bClone, sReplString, sXmlFile, xItem, xItemNew;
  xItemNew =
	<item>
	</item>

  sXmlFile = KLD_F.APP_NAME + ".xml";
  sInfo = wPalF.g10.eInfo.text;
  jNL = sInfo.indexOf (NL);                       // only 1st NL is used as separator.
  if (jNL < 0) {
   KLD_F.Message ("E", KLD_F.UItxt.ButtonSaveNew.@msg_01, KLD_F.APP_NAME);
   return;
   }
  sName = sInfo.substring(0, jNL);
  sInfo = sInfo.substring(jNL+1);
  if (sInfo == "") {
   KLD_F.Message ("E", KLD_F.UItxt.ButtonSaveNew.@msg_04, KLD_F.APP_NAME);
   return;
   }
  jItem = KLD_F.xSettings.item.length();
  xItemNew.name        = sName;
  xItemNew.info        = sInfo;
  xItemNew.findstring  = KLD_F.IsEmptyString (wPalF.g02.eFindWhat.text)  ? "--- empty / leer / vide ---" : wPalF.g02.eFindWhat.text;
  xItemNew.replstring  = KLD_F.IsEmptyString (wPalF.g06.eReplaceBy.text) ? "--- empty / leer / vide ---" : wPalF.g06.eReplaceBy.text
  xItemNew.findtype    = wPalF.g01.lstFindTypeF.selection;
  if (wPalF.g03.rbSimple.value)     {xItemNew.options.@searchMode = 0;}
  else if (wPalF.g03.rbWild.value)  {xItemNew.options.@searchMode = 1;}
  else                              {xItemNew.options.@searchMode = 2;}
  xItemNew.options.@word    = wPalF.g04.chkWord.value  ? 1 : 0;
  xItemNew.options.@case    = wPalF.g04.chkCase.value  ? 1 : 0;
  xItemNew.options.@back    = wPalF.g04.chkBack.value  ? 1 : 0;
  xItemNew.options.@clone   = wPalF.g04.chkClone.value ? 1 : 0;
  if (wPalF.g05.rbText.value)            {xItemNew.options.@replMode = 0;}
  else if (wPalF.g05.rbPaste.value)      {xItemNew.options.@replMode = 1;}
  else if (wPalF.g05.rbPasteNoFmt.value) {xItemNew.options.@replMode = 2;}
  else                                   {xItemNew.options.@replMode = 3;}
  KLD_F.xSettings.item[jItem]  = xItemNew;
  if (KLD_F.WriteXMLdata (KLD_F.xSettings, sXmlFile)) { // Recreate the file from XML object
    KLD_F.ConvertSelfClosingElements (sXmlFile);
    KLD_F.GetSettingsList ();                     // Recreate drop down settings list
    KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.ButtonSaveNew.@msg_02, sName), "black");
//  KLD_F.ClearInfoArea();                        // alternative to msg_02
  } else {
    KLD_F.DisplayInfoF (KLD_F.Message ("e", KLD_F.UItxt.ButtonSaveNew.@msg_03, sName), "red");
  }
} //--- end of ButtonSaveNew ----------------------------------------------------------------------

KLD_F.ClearInfoArea = function ()  { // ===========================================================
/*            Clear info area in the panel
Called by     PrepareBookProc, PalFindRepl, ButtonDelete, ButtonFind, ButtonReplace, SelectedInCatalogA,
              SelectedInCatalogB, SelectFindType
Calling       -
*/
  wPalF.g10.eInfo.text = "";
} //--- end of ClearInfoArea ----------------------------------------------------------------------

KLD_F.DisplayInfoF = function (sText, sColour) { // ===============================================
/*            Display text in the information area
Arguments     sText           The text to be displayed. It may contain line breaks (CTRL+Enter)
              sColour         "black" or "red" - other colours are not defined.
Returns       Function value is xxx
Called by     CheckForPotentialLoop, FindColourUse, FindInSelection, GetBookComponents, GetFindParameters,
              PrepareBookProc, RemoveOverride, WarnBackwardsSearch, PalFindRepl, ButtonDelete, ButtonFind,
              ButtonRegex, ButtonReplace, ButtonReplAll, ButtonRestore, ButtonSave, ButtonSaveNew,
              DisplaySettingsInfo, GetCatalogF, SelectedInCatalogA, SelectedInCatalogB, SelectFindType
Calling       -
History       2020-12-29
*/
  wPalF.g10.eInfo.text = sText;
  if (sColour == "red") {
    wPalF.g10.eInfo.graphics.foregroundColor = wPalF.g10.eInfo.graphics.newPen (wPalF.g10.eInfo.graphics.PenType.SOLID_COLOR, [1,0,0], 1);
    KLD_F.StdSound();
  } else {
    wPalF.g10.eInfo.graphics.foregroundColor = wPalF.g10.eInfo.graphics.newPen (wPalF.g10.eInfo.graphics.PenType.SOLID_COLOR, [0,0,0], 1);
  }
} //--- end of DisplayInfoF -----------------------------------------------------------------------

KLD_F.DisplaySettingsInfo = function () { // ======================================================
/*            Display the information form the XML object retrieved
Called by     PalFindRepl
Calling       ButtonRestore, DisplayInfoF
Comment       lstSettingName is a 'copy' of asSettingName. In the latter we have the orginal index appended
History       2020-12-05
*/
var indexOfTAB, info, jItem, jList, xItem;

  if (wPalF.g09.lstSettingName.selection == undefined) {return;} // just refreshing the list after save
  jList = wPalF.g09.lstSettingName.selection.index;
  if (jList == 0) {KLD_F.ButtonRestore(); return;}  // the blank item selected
  indexOfTAB = KLD_F.asSettingName[jList].indexOf("\t");  // starts with = ?
  jItem = KLD_F.asSettingName[jList].substr(indexOfTAB+1); // index in original XML file
  KLD_F.iSettings = jItem;                        // item index in settings XML file
  xItem = KLD_F.xSettings.item[jItem];
  KLD_F.sSettingName = xItem.name.toString();
  info = xItem.info.toString();                   // get the other properties similary
  if (info.charAt(0) == "!") {
    info = info.substr(1);
    KLD_F.DisplayInfoF (info, "red");
  } else {
    KLD_F.DisplayInfoF (info, "black");
  }
  wPalF.g09.btnRetrieve.enabled    = true;
} //--- end of DisplaySettingsInfo ----------------------------------------------------------------

KLD_F.FillFindTypeData = function () { // =========================================================
/*            Fill arrays for UI drop-down lists
Called by     PalFindRepl
Calling       -
Comment       With the transtion to UI texts in XML file these arrays can not be defined
              with global values.
History       2020-11-17
*/
var c0, c1, j, jComma, n, constant = [];
  KLD_F.asFindType.length = 0;                    // clear arrays
  KLD_F.asFType.length = 0;
  n = KLD_F.UItxt.asFindType.item.length();       // file types and related properties
  for (j = 0; j < n; j++) {
    KLD_F.asFindType.push (KLD_F.UItxt.asFindType.item[j].@type.toString());
    KLD_F.asFindModes.push(KLD_F.UItxt.asFindType.item[j].@Fmodes.toString());
    KLD_F.asReplModes.push(KLD_F.UItxt.asFindType.item[j].@Rmodes.toString());
    KLD_F.asFType.push    (KLD_F.UItxt.asFindType.item[j].@FType.toString()); // Mnemonic ID
  }
  KLD_F.iFindDefault = KLD_F.UItxt.iFindDefault.toString();
} //--- end of FillFindTypeData -------------------------------------------------------------------

KLD_F.GetCatalogAny = function () { // ============================================================
/*            To replace objects: fill the ancillary drop down list with items to be chosen from
Called by     PalFindRepl
Calling       GetCharFmtsList, FillDlgList, GetColourNameList, GetCondFmtsList, SetReplButtons,
              GetMarkerTypeList, GetObjFmtList, GetPgfFmtsList, GetTableFmtList, GetVarNameList,
              GetXrefFmtsList, Message
Comment       Generally, any catalgue may be empty and requires a test
              The filled list starts with an empty entry to handle Delte in ButtonReplace
History       2022-05-10
*/
var aoSourceVars = [], iFT, nUsrNames, sFType;

  iFT    = wPalF.g01.lstFindTypeF.selection.index;
  sFType = KLD_F.asFType[iFT];

   if        (sFType == "CHRTAG") {                // ------------------------------- Character tags
    KLD_F.GetCharFmtsList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    KLD_F.FillDlgList (wPalF.g06a.lstCatalgF, KLD_F.asCatalogF, true);
	} else if (sFType == "COLOUR") {                // --------------------------------- Colour names
    nUsrNames = KLD_F.GetColourNameList (KLD_F.oCurrentDoc, KLD_F.asCatalogC);
    KLD_F.FillDlgList (wPalF.g06a.lstCatalgF, KLD_F.asCatalogC, true);
    KLD_F.asCatalogF = KLD_F.asCatalogC;
    KLD_F.asCatalogF.splice(0, 0, "");            // for ListSelector; insert blank item
	} else if (sFType == "CONTAG") {                // ------------------------------ Condition names
    KLD_F.GetCondFmtsList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    KLD_F.FillDlgList (wPalF.g06a.lstCatalgF, KLD_F.asCatalogF, true);
	} else if (sFType == "EMTTXL") {                // ------------------------------- Empty Textline
	  KLD_F.SetReplButtons (true); // Skip requirement of selection
  } else if (sFType == "MRKANY" || sFType == "MRKTXT" || sFType == "MRKTYP") { // --------- Markers
    KLD_F.GetMarkerTypeList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    KLD_F.FillDlgList (wPalF.g06a.lstCatalgF, KLD_F.asCatalogF, true);
	} else if (sFType == "OSTTAG") {                // ---------------------- Object styles catalogue
    KLD_F.GetObjFmtList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    KLD_F.FillDlgList (wPalF.g06a.lstCatalgF, KLD_F.asCatalogF, true);
  } else if (sFType == "PGFTAG") {                // ------------------------------- Paragraph tags
    KLD_F.GetPgfFmtsList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    KLD_F.FillDlgList (wPalF.g06a.lstCatalgF, KLD_F.asCatalogF, true);
	} else if (sFType == "TBLANY"|| sFType == "TBLTAG") {  // -------------------------------- Tables
    KLD_F.GetTableFmtList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    KLD_F.FillDlgList (wPalF.g06a.lstCatalgF, KLD_F.asCatalogF, true);
	} else if (sFType == "TXTTXT" || sFType == "VARANY" || sFType == "VARNAM") { // ------- Variables
    nUsrNames = KLD_F.GetVarNameList (KLD_F.oCurrentDoc, KLD_F.asCatalogF, aoSourceVars);
    KLD_F.FillDlgList (wPalF.g06a.lstCatalgF, KLD_F.asCatalogF, true);
	} else if (sFType == "XRFANY" || sFType == "XRFFMT") { // ---------------------- Cross References
    KLD_F.GetXrefFmtsList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    KLD_F.FillDlgList (wPalF.g06a.lstCatalgF, KLD_F.asCatalogF, true);
  } else {
    alert (KLD_F.Message ("e", KLD_F.UItxt.General.@ProgramError, "GetCatalogAny")); // ogn errir
  }
} //--- end of GetCatalogAny ----------------------------------------------------------------------

KLD_F.GetCatalogF = function (sFType) { // ========================================================
/*            Fill the ancillary drop down list with items to be chosen from
Argument      sFType          Textual ID of Find Type. See Table asFindType
Called by     SelectFindType
Calling       GetCharFmtsList, DisplayInfoF, FillDlgList, GetColourNameList, GetCondFmtsList,
              GetMarkerTypeList, GetObjFmtList, GetPgfFmtsList, GetTableFmtList, GetVarNameList,
              GetXrefFmtsList
Comment       Generally, any catalgues may be empty and requires a test
History       2020-11-17
*/
var aoSourceVars = [], iFT, nUsrNames, sFType;

  iFT    = wPalF.g01.lstFindTypeF.selection.index;
  sFType = KLD_F.asFType[iFT];
  wPalF.g01.lstCatalgF.enabled  = false;          // all non-catalogue demanding entries

  if (sFType == "CHRTAG") {                       // ---  5 -------------------- character catalogue
    KLD_F.GetCharFmtsList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    if (KLD_F.asCatalogF.length == 0) {KLD_F.DisplayInfoF (KLD_F.UItxt.GetCatalogF.@msg_01, "red")};
    KLD_F.FillDlgList (wPalF.g01.lstCatalgF, KLD_F.asCatalogF);
    wPalF.g01.lstCatalgF.enabled  = true;
	} else if (sFType == "COLOUR") {                // --- 96 -------------------------- colour names
    nUsrNames = KLD_F.GetColourNameList (KLD_F.oCurrentDoc, KLD_F.asCatalogC);
    if (nUsrNames == 0) {KLD_F.DisplayInfoF (KLD_F.UItxt.GetCatalogF.@msg_09, "red")};
    KLD_F.FillDlgList (wPalF.g01.lstCatalgF, KLD_F.asCatalogC);
    KLD_F.asCatalogF = KLD_F.asCatalogC;  // for ListSelector
    wPalF.g01.lstCatalgF.enabled  = true;
	} else if (sFType == "CONTAG") {                // --- 13 ----------------------- condition names
    KLD_F.GetCondFmtsList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    if (KLD_F.asCatalogF.length == 0) {KLD_F.DisplayInfoF (KLD_F.UItxt.GetCatalogF.@msg_08, "red")};
    KLD_F.FillDlgList (wPalF.g01.lstCatalgF, KLD_F.asCatalogF);
    wPalF.g01.lstCatalgF.enabled  = true;
	} else if (sFType == "MRKTYP") {                // ---  9 -------------------------- marker types
    KLD_F.GetMarkerTypeList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    if (KLD_F.asCatalogF.length == 0) {KLD_F.DisplayInfoF (KLD_F.UItxt.GetCatalogF.@msg_03, "red")};
    KLD_F.FillDlgList (wPalF.g01.lstCatalgF, KLD_F.asCatalogF);
    wPalF.g01.lstCatalgF.enabled  = true;
	} else if (sFType == "OSTTAG") {                // ---- 18 ---------------object styles catalogue
    KLD_F.GetObjFmtList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    if (KLD_F.asCatalogF.length == 0) {KLD_F.DisplayInfoF (KLD_F.UItxt.GetCatalogF.@msg_04, "red")};
    KLD_F.FillDlgList (wPalF.g01.lstCatalgF, KLD_F.asCatalogF);
    wPalF.g01.lstCatalgF.enabled  = true;
	} else if (sFType == "PGFTAG") {                // ----  4 ------------------ paragraph catalogue
    KLD_F.GetPgfFmtsList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    if (KLD_F.asCatalogF.length == 0) {KLD_F.DisplayInfoF (KLD_F.UItxt.GetCatalogF.@msg_05, "red")};
    KLD_F.FillDlgList (wPalF.g01.lstCatalgF, KLD_F.asCatalogF);
    wPalF.g01.lstCatalgF.enabled  = true;
	} else if (sFType == "TBLTAG") {                // ----  6 ---------------------- table catalogue
    KLD_F.GetTableFmtList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    if (KLD_F.asCatalogF.length == 0) {KLD_F.DisplayInfoF (KLD_F.UItxt.GetCatalogF.@msg_06, "red")};
    KLD_F.FillDlgList (wPalF.g01.lstCatalgF, KLD_F.asCatalogF);
    wPalF.g01.lstCatalgF.enabled  = true;
	} else if (sFType == "VARNAM") {                // ---- 12 ------------------ variables catalogue
    nUsrNames = KLD_F.GetVarNameList (KLD_F.oCurrentDoc, KLD_F.asCatalogF, aoSourceVars);
    if (nUsrNames == 0) {KLD_F.DisplayInfoF (KLD_F.UItxt.GetCatalogF.@msg_07, "red")};
    KLD_F.FillDlgList (wPalF.g01.lstCatalgF, KLD_F.asCatalogF);
    wPalF.g01.lstCatalgF.enabled  = true;
	} else if (sFType == "XRFFMT") {                // ---- 11 ---------------cross reference formats
    KLD_F.GetXrefFmtsList (KLD_F.oCurrentDoc, KLD_F.asCatalogF);
    if (KLD_F.asCatalogF.length == 0) {KLD_F.DisplayInfoF (KLD_F.UItxt.GetCatalogF.@msg_02, "red")};
    KLD_F.FillDlgList (wPalF.g01.lstCatalgF, KLD_F.asCatalogF);
    wPalF.g01.lstCatalgF.enabled  = true;
  }
} //--- end of GetCatalogF ------------------------------------------------------------------------

KLD_F.SelectedInCatalogA = function () { // =======================================================
/*            Display properties of the selected objects (Xref-format, Variable name)
Returns       -
Called by     wPalF.g01.lstCatalgF.onChange
Calling       ClearInfoArea, DisplayInfoF
History       2020-12-17
*/
 var sName, sDef, oFmt;

  KLD_F.ClearInfoArea ();
  listIndex = wPalF.g01.lstFindTypeF.selection.index;
  sFType = KLD_F.asFType[listIndex];
  if (wPalF.g01.lstCatalgF.selection == null) {return}; // called before something is selected!
  wPalF.g02.eFindWhat.text    = wPalF.g01.lstCatalgF.selection;
  KLD_F.bFirstFind = true;                        // Find cycle starts

  if        (sFType == "MRKTYP") {                // display definition in info area
    sName   = wPalF.g02.eFindWhat.text;
    oMarker = KLD_F.oCurrentDoc.GetNamedMarkerType (sName);
    sDef    = oMarker.MarkerText;
    KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.SelectedInCatalogA.@msg_01, sName, sDef), "black")

  } else if (sFType == "VARNAM") {                // display definition in info area
    sName   = wPalF.g02.eFindWhat.text;
    oFmt    = KLD_F.oCurrentDoc.GetNamedVarFmt (sName);
    sDef    = oFmt.Fmt;
    KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.SelectedInCatalogA.@msg_02, sName, sDef), "black")

  } else if (sFType == "XRFFMT") {                // display definition in info area
    sName   = wPalF.g02.eFindWhat.text;
    oFmt    = KLD_F.oCurrentDoc.GetNamedXRefFmt (sName);
    sDef    = oFmt.Fmt;
    KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.SelectedInCatalogA.@msg_03, sName, sDef), "black")
  }
  wPalF.g08.btnFind.enabled   = true;
} //--- end of SelectedInCatalogA -----------------------------------------------------------------

KLD_F.SelectedInCatalogB = function () { // =======================================================
/*            Display properties of the selected objects (Marker, Table-fmt, Variable name, Xref-fmt)
Returns       -
Called by     wPalF.g06a.lstCatalgF.onChange
Calling       ClearInfoArea, IsOneOf, DisplayInfoF, SetReplButtons
History       2020-12-17
*/
var iFT, sName, sDef, oFmt;

  iFT    = wPalF.g01.lstFindTypeF.selection.index;
  sFType = KLD_F.asFType[iFT];

  KLD_F.ClearInfoArea ();
  if (wPalF.g06a.lstCatalgF.selection == null) {return}; // called before something is selected!

  if (KLD_F.IsOneOf ("CHRTAG COLOUR CONTAG OSTTAG OSTFMO PGFTAG TBLANY TBLTAG ", sFType) > 0) {
    wPalF.g06.eReplaceBy.text = wPalF.g06a.lstCatalgF.selection;
    if (wPalF.g06a.lstCatalgF.selection.index == 0) {sName = "";
    } else {
      sName   = wPalF.g06.eReplaceBy.text;
    }

  } else if (sFType == "MRKANY" || sFType == "MRKTYP") { // ---------------------Replacement Marker
    wPalF.g06.eReplaceBy.text = wPalF.g06a.lstCatalgF.selection;
    sName   = wPalF.g06.eReplaceBy.text;
    oMarker = KLD_F.oCurrentDoc.GetNamedMarkerType (sName);
    sDef    = oMarker.MarkerText;
    KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.SelectedInCatalogB.@msg_01, sName, sDef), "black")
    sName   = wPalF.g06.eReplaceBy.text;

  } else if (sFType == "TXTTXT" || sFType == "VARANY" || sFType == "VARNAM") { // Replacement variable
    wPalF.g06.eReplaceBy.text = wPalF.g06a.lstCatalgF.selection;
    sName   = wPalF.g06.eReplaceBy.text;
    oFmt    = KLD_F.oCurrentDoc.GetNamedVarFmt (sName);
    sDef    = oFmt.Fmt;
    KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.SelectedInCatalogB.@msg_02, sName, sDef), "black")

	} else if (sFType == "XRFANY" || sFType == "XRFFMT") { // -----------Replacement Cross Reference
    wPalF.g06.eReplaceBy.text = wPalF.g06a.lstCatalgF.selection;
    sName   = wPalF.g06.eReplaceBy.text;
    oFmt    = KLD_F.oCurrentDoc.GetNamedXRefFmt (sName);
    sDef    = oFmt.Fmt;
    KLD_F.DisplayInfoF (KLD_F.Message ("i", KLD_F.UItxt.SelectedInCatalogB.@msg_03, sName, sDef), "black")
  }
  KLD_F.oReplObject.type = sFType == "TXTTXT" ? "VARANY" : sFType; // --------- Replace by variable
  KLD_F.oReplObject.name = sName;
  KLD_F.oReplObject.def  = sDef;
  KLD_F.SetReplButtons (true);
} //--- end of SelectedInCatalogB -----------------------------------------------------------------

KLD_F.SelectFindType = function () {  // ==========================================================
/*            Find out which catalogue must be presented
Returns       -
Called by     PalFindRepl
Calling       ClearInfoArea, SetFindMode, SetReplMode, SetFindOptions, IsOneOf, DisplayInfoF,
              CollectUserStrings, GetCatalogF
Comment       It turns out that catalogues exist only for those items with no FV constant
              Only those items with FV = 8 have a secondary parameter FV
History       2020-12-17
*/
var listIndex, oDoc = KLD_F.oCurrentDoc, sFType;

  if (!KLD_F.bRetrieved) {
    KLD_F.ClearInfoArea ();                       // not to be done immed. after retrieve
  }
  KLD_F.bRetrieved = false;
  wPalF.g02.eFindWhat.text = "";
  wPalF.g06.eReplaceBy.text = "";
//wPalF.g09.lstSettingName.selection = 0;         // show blank entry
  if (wPalF.g01.lstFindTypeF.selection !== null) {// on restore
    listIndex = wPalF.g01.lstFindTypeF.selection.index;
  }
  sFType = KLD_F.asFType[listIndex];
  KLD_F.SetFindMode (listIndex);                  // Avoid dangerous actions
  KLD_F.SetReplMode (listIndex);                  // Avoid dangerous actions
  KLD_F.SetFindOptions (listIndex);               // Avoid useless selections
  wPalF.g01.lstCatalgF.selection = null;          // clear selection
  wPalF.g06a.lstCatalgF.enabled  = false;         // set true by GetCatalogF
  wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecials; // default, mostly dimmed
  KLD_F.bFirstFind = true;                        // Find cycle starts

  if (KLD_F.IsOneOf ("CHRFMT CONTXT TXTCLP ", sFType) > 0) { // ~~~~~~~~~ Not handled by FMfindRepl
    if (sFType == "CHRFMT" ) {                    // ---------------------------- Character Format:
      KLD_F.DisplayInfoF (KLD_F.UItxt.SelectFindType.@msg_01, "red");
    } else if (sFType == "CONTXT") {              // -------------------------- Conditional Text...
      KLD_F.DisplayInfoF (KLD_F.UItxt.SelectFindType.@msg_03, "red");
    } else if (sFType == "TXTCLP") {              // ---------------- Text on Clipboard (formatted)
      KLD_F.DisplayInfoF (KLD_F.UItxt.SelectFindType.@msg_02, "red");
    }
    wPalF.g08.btnFind.enabled   = false;
    return;
  }
//                                                ~~~~~~~~~~~~~~~ Not availabe in older FM versions
  if (KLD_F.IsOneOf ("OSTTAG OSTFMO ", sFType) > 0) {
    if (app.VersionMajor < 11) {KLD_F.DisplayInfoF (KLD_F.UItxt.SelectFindType.@msg_04, "red");
      wPalF.g08.btnFind.enabled   = false;
      return;
    }
  } else if (sFType == "USRSTR") {
    if (app.VersionMajor < 14) {KLD_F.DisplayInfoF (KLD_F.UItxt.SelectFindType.@msg_05, "red");
      wPalF.g08.btnFind.enabled   = false;
      return;
    }
  }
//                                                ~~~~ Set label of RB Special for these Find Types
  if (KLD_F.IsOneOf ("CHRTAG CHRFMO COLOUR CONTAG EMTTXL MRKANY MRKTXT MRKTYP OSTTAG OSTFMO PGFFMO PGFLCK PGFTAG TBLANY TBLFMO TBLTAG TXTTXT USRSTR VARANY VARNAM XRFANY XRFFMT", sFType) > 0) {

    if        (sFType == "CHRTAG") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsC;  // Replace Obj Format
    } else if (sFType == "COLOUR") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsE;  // Replace Colour
    } else if (sFType == "CONTAG") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsC;  // Replace Obj Format
    } else if (sFType == "EMTTXL") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsG;  // Delete Object
    } else if (sFType == "MRKANY") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsF;  // Change Marker Type
    } else if (sFType == "MRKTXT") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsB;  // Replace Obj Text
    } else if (sFType == "MRKTYP") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsF;  // Replace Obj Type
    } else if (sFType == "OSTTAG" || sFType == "PGFTAG") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsC;  // Replace Obj Format
    } else if (sFType == "PGFLCK") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsH;  // Unlock ¶
    } else if (sFType == "TBLANY" || sFType == "TBLTAG") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsC;  // Replace Obj Format
    } else if (sFType == "TXTTXT") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsD;  // Replace by Variable
    } else if (sFType == "USRSTR" ) {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsB;  // Replace Obj Text
      KLD_F.CollectUserStrings (oDoc, KLD_F.aoUserStrings);
    } else if (sFType == "VARANY" || sFType == "VARNAM") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsC;  // Replace Obj Format
    } else if (sFType == "XRFANY" || sFType == "XRFFMT") {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsC;  // Replace Obj Format
    } else {
      wPalF.g05.rbSpecials.text = KLD_F.UItxt.wPalF.@g05rbSpecialsA;  // Remove Override
    }
  }
//                                                ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Other standard types
  KLD_F.GetCatalogF (sFType);
  wPalF.g08.btnFind.enabled   = true;
  return;
} //--- end of SelectFindType ---------------------------------------------------------------------

KLD_F.SelectSearchMode = function () { // =========================================================
/*            Define the valid checkboxes for selected find radio button
Called by     PalFindRepl
Calling       -
History       2020-12-05
*/
  KLD_F.ClearInfoArea ();
  wPalF.g04.chkCase.enabled    = true;            // always allowed
  if (wPalF.g03.rbRegEx.value) {                  // regex
    wPalF.g03.btnRegex.enabled = true;
    wPalF.g04.chkWord.enabled  = false;
//  wPalF.g04.chkWord.value    = false;
    wPalF.g04.chkBack.enabled  = false;
//  wPalF.g04.chkBack.value    = false;
  } else {                                        // simple search | wild cards
    wPalF.g03.btnRegex.enabled = false;
    wPalF.g04.chkWord.enabled  = true;
    wPalF.g04.chkBack.enabled  = true;
  }
    KLD_F.bFirstFind = true;                      // Find cycle starts
} //--- end of SelectSearchMode -------------------------------------------------------------------

KLD_F.SetActionButtons = function () { // =========================================================
/*            Set the action buttons for working on the document
Called by     ButtonRetrieve
Calling       -
History       2020-12-05
*/
  wPalF.g08.btnFind.enabled    = true;
  wPalF.g08.btnReplace.enabled = true;
  wPalF.g08.btnReplFnd.enabled = true;
  wPalF.g08.btnReplAll.enabled = true;
} //--- end of SetActionButtons -------------------------------------------------------------------

KLD_F.SetFindMode = function (listIndex) { // =====================================================
/*            Set the allowable find methods depending on the FindType
Arguments     listIndex       index of the selected item in the FindType list
Returns       -
Called by     SelectFindType
Calling       -
History       2020-12-23
*/
var aFmodes, sFindModes;

  sFindModes = KLD_F.asFindModes[listIndex];
  aFmodes = sFindModes.split(",");                // get bSimple, bWild, bRegex as 0|1
  wPalF.g03.rbSimple.enabled = aFmodes[1] == "T" ? true : false;
  wPalF.g03.rbWild.enabled   = aFmodes[2] == "T" ? true : false;
  wPalF.g03.rbRegEx.enabled  = aFmodes[3] == "T" ? true : false;
  wPalF.g07.rbSelect.enabled = aFmodes[4] == "T" ? true : false;
// set the default
  wPalF.g03.rbSimple.value   = aFmodes[0] == 1 ? true : false;
  wPalF.g03.rbWild.value     = aFmodes[0] == 2 ? true : false;
  wPalF.g03.rbRegEx.value    = aFmodes[0] == 3 ? true : false;
  wPalF.g07.rbSelect.value   = aFmodes[4] == 4 ? true : false; // should not be used
} //--- end of SetFindMode ------------------------------------------------------------------------

KLD_F.SetFindOptions = function (listIndex) { // ==================================================
/*            Enable the check boxes depending on the FindType
Arguments     listIndex       index of the selected item in the FindType list
Returns       -
Called by     SelectFindType
Calling       -
History       2020-12-30
*/
var sFoptions, aFoptions;

  sFoptions = KLD_F.UItxt.asFindType.item[listIndex].@Foptions.toString();
  aFoptions = sFoptions.split(",");               // get Case, Word, Back, Clone as 0|1

  wPalF.g04.chkCase.enabled  = aFoptions[0] == "T" ? true : false;
  wPalF.g04.chkWord.enabled  = aFoptions[1] == "T" ? true : false;
  wPalF.g04.chkBack.enabled  = aFoptions[2] == "T" ? true : false;
  wPalF.g04.chkClone.enabled = aFoptions[3] == "T" ? true : false;
// Default: none are set
  wPalF.g04.chkCase.value  = false;
  wPalF.g04.chkWord.value  = false;
  wPalF.g04.chkBack.value  = false;
  wPalF.g04.chkClone.value = false;
} //--- end of SetFindOptions ---------------------------------------------------------------------

KLD_F.SetReplButtons = function (state) { // ======================================================
/*            enable/disable Replacement buttons
Arguments     state           true for enable, false for disable
Returns       -
Called by     PalFindRepl, ButtonFind, ButtonReplace, GetCatalogAny, SelectedInCatalogB
Calling       -
History       2022-04-26
*/
  wPalF.g08.btnReplace.enabled = state;
  wPalF.g08.btnReplFnd.enabled = state;
  wPalF.g08.btnReplAll.enabled = state;
} //--- end of SetReplButtons ---------------------------------------------------------------------

KLD_F.SetReplMode = function (listIndex) { // =====================================================
/*            Set allowable ReplMode radios methods depending on the FindType
Arguments     listIndex       index of the selected item in the FindType list
Returns       -
Called by     SelectFindType
Calling       -
History       2020-12-28
*/
var aRmodes, sReplModes;

  sReplModes = KLD_F.asReplModes[listIndex];
  aRmodes = sReplModes.split(",");                // get bText, ... bSpecials as 0|1
  wPalF.g05.rbText.enabled       = aRmodes[1] == "T" ? true : false;
  wPalF.g05.rbPaste.enabled      = aRmodes[2] == "T" ? true : false;
  wPalF.g05.rbPasteNoFmt.enabled = aRmodes[3] == "T" ? true : false;
  wPalF.g05.rbSpecials.enabled   = aRmodes[4] == "T" ? true : false;
// set the default
  wPalF.g05.rbText.value         = aRmodes[0] == 1 ? true : false;
  wPalF.g05.rbPaste.value        = aRmodes[0] == 2 ? true : false;
  wPalF.g05.rbPasteNoFmt.value   = aRmodes[0] == 3 ? true : false;
//wPalF.g05.rbSpecials.value     = aRmodes[0] == 4 ? true : false;   // No others can be set
} //--- end of SetReplMode ------------------------------------------------------------------------

KLD_F.SettingsToPasteBuffer = function () { // ====================================================
/*            Collect the settings into paragraphs which can be formatted as wide list.
Comment       This does not work in book! - check already in button Retrieve
              NL must be inserted as String.fromCharCode(10);
              TAB must be inserted as String.fromCharCode(08);
Comment       Fixed parts of the collected text are localised.
Called by     ButtonRetrieve
Calling       CopyToClipboard
History       2021-03-22
*/
var sOptions, sText = "", NL, TAB, SEP = ", ";

  NL  = String.fromCharCode(10);
  TAB = String.fromCharCode(08);                  // FM tab, not ASCII tab!
  sText = sText + wPalF.g09.lstSettingName.selection.text + NL;
  sText = sText + "Info" + TAB + wPalF.g10.eInfo.text + NL;
  sText = sText + KLD_F.UItxt.wPalF.@g01t1 + TAB + wPalF.g01.lstFindTypeF.selection.text + NL;
  sText = sText + KLD_F.UItxt.wPalF.@g02t1 + TAB + wPalF.g02.eFindWhat.text + NL;
  sOptions = KLD_F.UItxt.wPalF.@g03t1 + TAB;
  if (wPalF.g03.rbSimple.value) {sOptions = sOptions + KLD_F.UItxt.wPalF.@g03rbSimple + SEP;}
  if (wPalF.g03.rbWild.value)   {sOptions = sOptions + KLD_F.UItxt.wPalF.@g03rbWild   + SEP;}
  if (wPalF.g03.rbRegEx.value)  {sOptions = sOptions + KLD_F.UItxt.wPalF.@g03rbRegEx  + SEP;}
  if (wPalF.g04.chkCase.value)  {sOptions = sOptions + KLD_F.UItxt.wPalF.@g04chkCase  + SEP;}
  if (wPalF.g04.chkWord.value)  {sOptions = sOptions + KLD_F.UItxt.wPalF.@g04chkWord  + SEP;}
  if (wPalF.g04.chkBack.value)  {sOptions = sOptions + KLD_F.UItxt.wPalF.@g04chkBack  + SEP;}
  sText = sText + sOptions + NL;
  sText = sText + KLD_F.UItxt.wPalF.@g06t1 + TAB + wPalF.g06.eReplaceBy.text + NL;
  KLD_F.CopyToClipboard(sText);
} //--- end of SettingsToPasteBuffer --------------------------------------------------------------
