﻿/* FMfindRepl.jsx ====== UTF-8 ====================================================================
              Extended find and replace palette
Location      E:\_DDDprojects\FM-FindRepl\InstallThis\FMfindRepl\FMfindRepl.pdf
Website       https://www.daube.ch/docu/fmaker86.html
Comment       XML objects require a toString() to get the string value of element or attribute
              This can be left out for ScriptUI objects, but not for many other situations.
              Hence I use this liberty only for ScriptUI objects.
Copyleft      2020 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       2020-09-28 started
              2020-11-04 debugging switched to Visual Studio Code + ExtendScript Debugger
              2020-11-17 UI text handling converted to XML methods
              2020-11-20 Implementation simulated and menu tested
              2021-01-06 No possibility to get the RegEx variables $1…$9 lets me stop this route
                         and try to hand over the settings to the FM F/C panel
              2021-03-25 Version 0.80 published on to web
              2021-05-31 Version 0.90 published on to web
              2021-12-06 Stating work on v1.00
              2022-11-25 v1.00 Published as my (80th) birthday gift to the community
              2022-12-31 v1.10 Published (Find locked PP added)
Analysis      2022-10-30 by E:\_DDDprojects\FM-JsxLib\TestFiles,Pgms\_Project-analysis.jsx
              Modules          6 + 3 language modules
              Functions      127
              Pgm lines     3433
              Comment lines 1483
              Empty lines    386
              Input lines   5302
              h spent 2022   365  until 2022-11-15
              Number of Modules =	6
*/ ; // ===========================================================================================

//@target framemaker

if (typeof KLD_F == "undefined") {
  KLD_F = {};                                     // Global script object for this suite
}
var KLD_F_lastNote = undefined;                   // Must not be in name-space object
//@include FMfindRepl\FMfindRepl_Globals.jsx      // all declarations, includes FMfindRepl\FMfindRepl_Texts.jsx
//@include FMfindRepl\FMfindRepl_Ancillary.jsx    // ancillary functions
//@include FMfindRepl\FMfindRepl_DocObj.jsx       // functions concerning document objects
//@include FMfindRepl\FMfindRepl_PalF.jsx         // incl. button functions etc.
//@include FMfindRepl\FMfindRepl_PalR.jsx         // incl. button functions etc.
//         FMfindRepl\FMfindRepl_de.xml           // dialog/message text read by SwitchUIlanguageX
//         FMfindRepl\FMfindRepl_en.xml           // dialog/message text read by SwitchUIlanguageX
//         FMfindRepl\FMfindRepl_fr.xml           // dialog/message text read by SwitchUIlanguageX

KLD_F.Main = function () { //=== ==================================================================
/*            Initial function of this script suite.
Called by     FM runtime
Calling       DebugMenu, SwitchUIlanguageX, SetupFMfindRepl, SetupNotifications, SetUpMenus
Attention     With functions as objects the call to Main must be at end of this module.
History       2020-12-29
*/
var bIsOK;
  Console (KLD_F.APP_NAME + " v" + KLD_F.APP_VERSION);
  KLD_F.APP_LANG = KLD_F.SwitchUIlanguageX (KLD_F.APP_NAME); // etablish the UI texts
  bIsOK = KLD_F.SetupFMfindRepl ();            // get inital data / terminate on error
  if (!bIsOK) {return false;}
// --- For debugging you need to switch statements (ESTK not working when menus active)
//                              real work       debugging
  KLD_F.SetupNotifications ();    // uncomment    comment
  KLD_F.SetUpMenus();             // uncomment    comment
//KLD_F.DoFindReplace ();         // comment      uncomment  Directly call the panel
} ; // --- end Main  ------------------------------------------------------------------------------

KLD_F.SetUpMenus = function () { //=== Define the menu items for document and book ================
/*            Set up the menu and short cuts
Called by     Main
Calling       -
Attention     This function MUST NOT be named DefineMenu - because this is a reserved function!
Comment       There is no menu "Special" in books => EditMenu used instead for both cases
              For a list of DDD used Esc shortcuts see E:\_DDDprojects\FM-GUI-modifications\ESC-sequences\FM-ESC-sequences.fm
              The command location in the menu is only observed in FM-16 upwards.
Reference     Klaus Göbel at https://forums.adobe.com/thread/2236201
              (My method with kb-labels after \t in the menu entry defintions does not work in FM-14)
History       2020-11-20
*/
var menuLocation, nearbyCmd, oCmd = {}, sc01, sc02;
// --- Assign commands to menus
  menuLocation = app.GetNamedMenu("EditMenu");
  if (KLD_F.FM_ShortCut) {
    sc02 = KLD_F.UItxt.SetUpMenus.@sc02.toString();
    oCmd.FindInDoc  = menuLocation.DefineAndAddCommand(1,"FMfindRepl_Doc", KLD_F.UItxt.SetUpMenus.@menu02 + " v" + KLD_F.APP_VERSION, sc02);
    oCmd.FindInDoc.KeyboardShortcutLabel  = KLD_F.UItxt.SetUpMenus.@scl02;
  } else {
    sc01 = KLD_F.UItxt.SetUpMenus.@sc01.toString();
    oCmd.FindInDoc  = menuLocation.DefineAndAddCommand(1,"FMfindRepl_Doc", KLD_F.UItxt.SetUpMenus.@menu01 + " v" + KLD_F.APP_VERSION, sc01);
    oCmd.FindInDoc.KeyboardShortcutLabel  = KLD_F.UItxt.SetUpMenus.@scl01;
  }
  nearbyCmd = app.GetNamedCommand("FindNext");
  oCmd.FindInDoc.PrevMenuItemInMenu = nearbyCmd;

  menuLocation = app.GetNamedMenu("BookEditMenu");
  if (KLD_F.FM_ShortCut) {
    sc02 = KLD_F.UItxt.SetUpMenus.@sc02.toString();
    oCmd.FindInBk  = menuLocation.DefineAndAddCommand(1,"FMfindRepl_Bk", KLD_F.UItxt.SetUpMenus.@menu02 + " v" + KLD_F.APP_VERSION, sc02);
    oCmd.FindInBk.KeyboardShortcutLabel  = KLD_F.UItxt.SetUpMenus.@scl02;
  } else {
    sc01 = KLD_F.UItxt.SetUpMenus.@sc01.toString();
    oCmd.FindInBk  = menuLocation.DefineAndAddCommand(1,"FMfindRepl_Bk", KLD_F.UItxt.SetUpMenus.@menu01 + " v" + KLD_F.APP_VERSION, sc01);
    oCmd.FindInBk.KeyboardShortcutLabel  = KLD_F.UItxt.SetUpMenus.@scl01;
  }
  nearbyCmd = app.GetNamedCommand("FindNext");
  oCmd.FindInBk.PrevMenuItemInMenu = nearbyCmd;

  UpdateMenus();                                  // refresh and actually make the new menu appear
} //--- end SetUpMenus ----------------------------------------------------------------------------

function Command (cmd) { // =======================================================================
/*            Assign functions to menu commands
Arguments     cmd            reference number of command from the menu definition
Returns       -
Called by     FM runtime
Calling       DoFindReplace
Attention     This functions can only have this name (Command) and must use the classic syntax
              function Command (cmd) --- not --- Command = function (cmd) !
History       2020-11-19 tested 'as implemented'
*/
  switch (cmd) {
    case 1:
      KLD_F.DoFindReplace();
      break;
  }
} //--- end Command -------------------------------------------------------------------------------

KLD_F.DoFindReplace = function () { //=============================================================
/*            Handle the task of this script for documents
Arguments     -
Returns       -
Called by     Command, Main
Calling       PalFindRepl, SaveCurrentLocation
History       2020-12-29
*/
var  oLocation;

  if (app.ActiveBook.ObjectValid()) {
    KLD_F.oCurrentBook = app.ActiveBook;
    KLD_F.bDocActive = false;
    KLD_F.bBookActive = true;
  } else if (app.ActiveDoc.ObjectValid()) {
    KLD_F.oCurrentDoc = app.ActiveDoc;
    KLD_F.bDocActive = true;
    KLD_F.bBookActive = false;
    oLocation = KLD_F.SaveCurrentLocation(KLD_F.oCurrentDoc); // Save cursor location/TR
  } else {
//  This path is never entered: a shortcut or menu item executes only if a book or doc is active
//  KLD_F.Message ("E", KLD_F.UItxt.General.@BookDocRequired,  "DoFindReplace");
  }
    KLD_F.PalFindRepl ();
    if (KLD_F.bDocActive)  {wPalF.g07.rbDoc.value = true;  wPalF.g07.rbBook.value = false;}
    if (KLD_F.bBookActive) {wPalF.g07.rbDoc.value = false; wPalF.g07.rbBook.value = true;}
} //--- end DoFindReplace -------------------------------------------------------------------------

KLD_F.SetupFMfindRepl = function () { //===========================================================
/*            Initialise script
Returns       -
Called by     Main
Calling       GetScriptPath
History       2020-12-29
*/
var sHelpFile;
  KLD_F.sScriptDir  = KLD_F.GetScriptPath () + "\\" + KLD_F.APP_NAME;
  sHelpFile         = KLD_F.sScriptDir + "\\" + KLD_F.APP_NAME + ".pdf"
  KLD_F.oHelpFile   = File (sHelpFile);
  return true;
} //--- end SetupFMfindRepl -----------------------------------------------------------------------

KLD_F.SetupNotifications = function () { //========================================================
/*            Set up the watch dogs
Called by     Main
Calling       -
Comment       Script Registration and unregistration:
              The script will be registered if it contains SetupNotifications and Notify functions
              It stays registered until it is unregistred. When the RemoveNotification is not
              called at close of FM (because the script just has been killed), the registration still
              exists - even across sessions!
              Hence during development manual unregistration is necessary before each modification
              and restart of the script.
              This makes debugging cumbersome - hence I switch off this function during debugging
*/
  Notification (Constants.FA_Note_PostActiveDocChange, true);      // 105
  Notification (Constants.FA_Note_PreQuitSession, true);           //  26
} //--- end SetupNotifications --------------------------------------------------------------------

KLD_F.RemoveNotifications = function () { //=======================================================
/*            Remove the watch dogs
Called by     Notify
Calling       -
Reference     Klaus Göbel in https://forums.adobe.com/thread/2174148
              Markus WiedeMaier in https://forums.adobe.com/thread/2276988
*/
  Notification (Constants.FA_Note_PostActiveDocChange, false);
  Notification (Constants.FA_Note_PreQuitSession, false);
} //--- end RemoveNotifications -------------------------------------------------------------------

function Notify (iNote, object, sParam, iParam) {  //==============================================
/*            Handle triggered events
Arguments     iNote           notification id to identify the event.
              object          current document or book object which generated the event.
              sParam          string, if any, associated with the notification.
              iParam          Integer, if any, associated with the notification. Probably the fcode or bit flags.
Called by     FM runtime
Calling       PrepareBookProc, RemoveNotifications, Message
Attention     • This function must be named Notify and it can not set up as object
Comment       • See documentation concerning Notitfy in FMjsxLib
Reference     F_ApiNotification() in FDK Function Ref.
Used in       FMfindRepl
History       2020-12-22 and later
*/

//Console ("Notify iNote = " + iNote + "   obj " + object.constructor.name);
  switch (iNote) {
// ---------------------------------------------- 105 active document has changed
    case Constants.FA_Note_PostActiveDocChange:
//Console ("---→ " + typeof wPalF);
      if (object.ObjectValid()) {                 // change of document
        KLD_F.oCurrentDoc = object;
        KLD_F.bDocActive = true;
        KLD_F.bBookActive = false;
        KLD_F.bFirstFind  = true;                 // restart find cycles
        KLD_F.LastDocID = object.id
        if (typeof wPalF == "object") {
          wPalF.g07.rbDoc.value = true;
          wPalF.g07.rbBook.value = false;
        }
      } else if (KLD_F_lastNote == 105 && !object.ObjectValid()) { // switch from doc → book window
        KLD_F.oCurrentBook = app.ActiveBook;
        KLD_F.bDocActive = false;
        KLD_F.bBookActive = true;
        if (typeof wPalF == "object") {
          wPalF.g07.rbDoc.value = false;
          wPalF.g07.rbBook.value = true;
          if (wPalF.visible) {
            KLD_F.PrepareBookProc ();
          }
        }
      }
      break;
// ---------------------------------------------- 26 just before quitting FM
    case Constants.FA_Note_PreQuitSession:
      KLD_F.RemoveNotifications ();
      break;
// ---------------------------------------------- unknown notification
    default:
      Console (KLD_F.Message ("e", KLD_F.UItxt.Notify.@msg_01, iNote));
  }
  KLD_F_lastNote = iNote;                         // must not be in name space object!
} //--- end Notify --------------------------------------------------------------------------------

KLD_F.GetScriptPath = function () { //=============================================================
/*            Get the script path
Returns       Function value is Path of the script
Called by     SetupFMfindRepl, OpenPdfAtDestination
Calling       -
Attention     If sub-scripts are in a sub-directory then this function must be in the main module
History       2020-12-29
*/
var lastBSlash, sScriptPath, sWinPath;

  sScriptPath = $.fileName;                       // script path (including the file-name of the script)
                              // platform independent format e.g. /e/_DDDprojects/FM+EN-escript/escript
  sWinPath = new File (sScriptPath);
  sScriptPath = sWinPath.fsName;                  // get Windows format
  lastBSlash = sScriptPath.lastIndexOf("\\");     // \ needs escaping
  sScriptPath = sScriptPath.substring(0, lastBSlash);
  return sScriptPath;                             // no final \ !
} //--- end GetScriptPath -------------------------------------------------------------------------

KLD_F.Main ();                                    // follow the standard …
